// Generated from player/lua/commands.lua

"--[[\n"
"This file is part of mpv.\n"
"\n"
"mpv is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU Lesser General Public\n"
"License as published by the Free Software Foundation; either\n"
"version 2.1 of the License, or (at your option) any later version.\n"
"\n"
"mpv is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU Lesser General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU Lesser General Public\n"
"License along with mpv.  If not, see <http://www.gnu.org/licenses/>.\n"
"]]\n"
"\n"
"local options = {\n"
"    persist_history = false,\n"
"    history_path = \"~~state/command_history.txt\",\n"
"}\n"
"\n"
"local input = require \"mp.input\"\n"
"local utils = require \"mp.utils\"\n"
"\n"
"local styles = {\n"
"    -- Colors are stolen from base16 Eighties by Chris Kempson\n"
"    -- and converted to BGR as is required by ASS.\n"
"    -- 2d2d2d 393939 515151 697374\n"
"    -- 939fa0 c8d0d3 dfe6e8 ecf0f2\n"
"    -- 7a77f2 5791f9 66ccff 99cc99\n"
"    -- cccc66 cc9966 cc99cc 537bd2\n"
"\n"
"    debug = \"{\\\\1c&Ha09f93&}\",\n"
"    v = \"{\\\\1c&H99cc99&}\",\n"
"    warn = \"{\\\\1c&H66ccff&}\",\n"
"    error = \"{\\\\1c&H7a77f2&}\",\n"
"    fatal = \"{\\\\1c&H5791f9&}\",\n"
"    completion = \"{\\\\1c&Hcc99cc&}\",\n"
"}\n"
"for key, style in pairs(styles) do\n"
"    styles[key] = style .. \"{\\\\3c&H111111&}\"\n"
"end\n"
"\n"
"local terminal_styles = {\n"
"    debug = \"\\027[90m\",\n"
"    v = \"\\027[32m\",\n"
"    warn = \"\\027[33m\",\n"
"    error = \"\\027[31m\",\n"
"    fatal = \"\\027[91m\",\n"
"}\n"
"\n"
"local platform = mp.get_property(\"platform\")\n"
"local path_separator = platform == \"windows\" and \"\\\\\" or \"/\"\n"
"local completion_pos\n"
"local completion_append\n"
"local last_text\n"
"local last_cursor_position\n"
"\n"
"local cache = {}\n"
"\n"
"\n"
"local function get_commands()\n"
"    if cache[\"commands\"] then\n"
"        return cache[\"commands\"]\n"
"    end\n"
"\n"
"    cache[\"commands\"] = mp.get_property_native(\"command-list\")\n"
"    table.sort(cache[\"commands\"], function(c1, c2)\n"
"        return c1.name < c2.name\n"
"    end)\n"
"\n"
"    return cache[\"commands\"]\n"
"end\n"
"\n"
"local function help_command(param)\n"
"    local output = \"\"\n"
"    if param == \"\" then\n"
"        output = \"Available commands:\\n\"\n"
"        for _, cmd in ipairs(get_commands()) do\n"
"            output = output  .. \"  \" .. cmd.name\n"
"        end\n"
"        output = output .. \"\\n\"\n"
"        output = output .. 'Use \"help command\" to show information about a command.\\n'\n"
"        output = output .. \"ESC or Ctrl+d exits the console.\\n\"\n"
"    else\n"
"        local cmd = nil\n"
"        for _, curcmd in ipairs(get_commands()) do\n"
"            if curcmd.name:find(param, 1, true) then\n"
"                cmd = curcmd\n"
"                if curcmd.name == param then\n"
"                    break -- exact match\n"
"                end\n"
"            end\n"
"        end\n"
"\n"
"        if not cmd then\n"
"            input.log('No command matches \"' .. param .. '\"!', styles.error,\n"
"                      terminal_styles.error)\n"
"            return\n"
"        end\n"
"\n"
"        output = output .. 'Command \"' .. cmd.name .. '\"\\n'\n"
"\n"
"        for _, arg in ipairs(cmd.args) do\n"
"            output = output .. \"    \" .. arg.name .. \" (\" .. arg.type .. \")\"\n"
"            if arg.optional then\n"
"                output = output .. \" (optional)\"\n"
"            end\n"
"            output = output .. \"\\n\"\n"
"        end\n"
"\n"
"        if cmd.vararg then\n"
"            output = output .. \"This command supports variable arguments.\\n\"\n"
"        end\n"
"    end\n"
"\n"
"    input.log(output:sub(1, -2))\n"
"end\n"
"\n"
"local function submit(line)\n"
"    -- match \"help [<text>]\", return <text> or \"\", strip all whitespace\n"
"    local help = line:match(\"^%s*help%s+(.-)%s*\044\") or\n"
"                 (line:match(\"^%s*help\044\") and \"\")\n"
"\n"
"    if help then\n"
"        help_command(help)\n"
"    elseif line ~= \"\" then\n"
"        mp.command(line)\n"
"    end\n"
"end\n"
"\n"
"local function opened()\n"
"    mp.enable_messages(\"terminal-default\")\n"
"end\n"
"\n"
"local function closed(text, cursor_position)\n"
"    mp.enable_messages(\"silent:terminal-default\")\n"
"\n"
"    last_text = text\n"
"    last_cursor_position = cursor_position\n"
"end\n"
"\n"
"local function command_list()\n"
"    local cmds = {}\n"
"    for i, command in ipairs(get_commands()) do\n"
"        cmds[i] = command.name\n"
"    end\n"
"\n"
"    return cmds\n"
"end\n"
"\n"
"local function property_list()\n"
"    if cache[\"property-list\"] then\n"
"        return cache[\"property-list\"]\n"
"    end\n"
"\n"
"    cache[\"property-list\"] = mp.get_property_native(\"property-list\")\n"
"\n"
"    for _, sub_property in pairs({\"video\", \"audio\", \"sub\", \"sub2\"}) do\n"
"        table.insert(cache[\"property-list\"], \"current-tracks/\" .. sub_property)\n"
"    end\n"
"\n"
"    for _, sub_property in pairs({\"text\", \"text-primary\"}) do\n"
"        table.insert(cache[\"property-list\"], \"clipboard/\" .. sub_property)\n"
"    end\n"
"\n"
"    return cache[\"property-list\"]\n"
"end\n"
"\n"
"local function profile_list()\n"
"    local profiles = {}\n"
"\n"
"    for i, profile in ipairs(mp.get_property_native(\"profile-list\")) do\n"
"        profiles[i] = profile.name\n"
"    end\n"
"\n"
"    return profiles\n"
"end\n"
"\n"
"local function option_info(option, prop, default)\n"
"    local key = \"option-info/\" .. option\n"
"    cache[key] = cache[key] or mp.get_property_native(key, false)\n"
"\n"
"    if not cache[key] then\n"
"        return default\n"
"    end\n"
"\n"
"    if prop then\n"
"        if cache[key][prop] ~= nil then\n"
"            return cache[key][prop]\n"
"        end\n"
"        return default\n"
"    end\n"
"\n"
"    return cache[key]\n"
"end\n"
"\n"
"local function list_option_list()\n"
"    if cache[\"option-list\"] then\n"
"        return cache[\"option-list\"]\n"
"    end\n"
"    cache[\"option-list\"] = {}\n"
"\n"
"    -- Don't log errors for renamed and removed properties.\n"
"    -- (Just mp.enable_messages(\"fatal\") still logs them to the terminal.)\n"
"    local msg_level_backup = mp.get_property(\"msg-level\")\n"
"    mp.set_property(\"msg-level\", msg_level_backup == \"\" and \"cplayer=no\"\n"
"                                 or msg_level_backup .. \",cplayer=no\")\n"
"\n"
"    for _, option in pairs(mp.get_property_native(\"options\")) do\n"
"        if option_info(option, \"type\", \"\"):find(\" list\044\") then\n"
"            table.insert(cache[\"option-list\"], option)\n"
"        end\n"
"    end\n"
"\n"
"    mp.set_property(\"msg-level\", msg_level_backup)\n"
"\n"
"    return cache[\"option-list\"]\n"
"end\n"
"\n"
"local function list_option_action_list(option)\n"
"    local type = option_info(option, \"type\")\n"
"\n"
"    if type == \"Key/value list\" then\n"
"        return {\"add\", \"append\", \"set\", \"remove\"}\n"
"    end\n"
"\n"
"    if type == \"String list\" or type == \"Object settings list\" then\n"
"        return {\"add\", \"append\", \"clr\", \"pre\", \"set\", \"remove\", \"toggle\"}\n"
"    end\n"
"end\n"
"\n"
"local function list_option_value_list(option)\n"
"    local values = mp.get_property_native(option)\n"
"\n"
"    if type(values) ~= \"table\" then\n"
"        return\n"
"    end\n"
"\n"
"    if type(values[1]) ~= \"table\" then\n"
"        return values\n"
"    end\n"
"\n"
"    for i, value in ipairs(values) do\n"
"        values[i] = value.label and \"\100\" .. value.label or value.name\n"
"    end\n"
"\n"
"    return values\n"
"end\n"
"\n"
"local function has_file_argument(candidate_command)\n"
"    local key = \"command-file/\" .. candidate_command\n"
"    if cache[key] ~= nil then\n"
"        return cache[key]\n"
"    end\n"
"\n"
"    for _, command in pairs(get_commands()) do\n"
"        if command.name == candidate_command then\n"
"            cache[key] = command.args[1] and\n"
"                         (command.args[1].name == \"filename\" or command.args[1].name == \"url\")\n"
"            return cache[key]\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"local function file_list(directory)\n"
"    if directory == \"\" then\n"
"        directory = \".\"\n"
"    else\n"
"        directory = mp.command_native({\"expand-path\", directory})\n"
"    end\n"
"\n"
"    local files = utils.readdir(directory, \"files\") or {}\n"
"\n"
"    for _, dir in pairs(utils.readdir(directory, \"dirs\") or {}) do\n"
"        files[#files + 1] = dir .. path_separator\n"
"    end\n"
"\n"
"    return files\n"
"end\n"
"\n"
"local function handle_file_completion(before_cur)\n"
"    local directory, last_component_pos =\n"
"        before_cur:sub(completion_pos):match(\"(.-)()[^\" .. path_separator ..\"]*\044\")\n"
"\n"
"    completion_pos = completion_pos + last_component_pos - 1\n"
"\n"
"    -- Don\"t use completion_append for file completion to not add quotes after\n"
"    -- directories whose entries you may want to complete afterwards.\n"
"    completion_append = \"\"\n"
"\n"
"    return file_list(directory)\n"
"end\n"
"\n"
"local function handle_choice_completion(option, before_cur)\n"
"    local info = option_info(option, nil, {})\n"
"\n"
"    if info.type == \"Flag\" then\n"
"        return { \"no\", \"yes\" }, before_cur\n"
"    end\n"
"\n"
"    if info[\"expects-file\"] then\n"
"        return handle_file_completion(before_cur)\n"
"    end\n"
"\n"
"    -- Fix completing the empty value for --dscale and --cscale.\n"
"    if info.choices and info.choices[1] == \"\" and completion_append == \"\" then\n"
"        info.choices[1] = '\"\"'\n"
"    end\n"
"\n"
"    return info.choices\n"
"end\n"
"\n"
"local function command_flags_at_1st_argument_list(command)\n"
"    local flags = {\n"
"        [\"playlist-next\"] = {\"weak\", \"force\"},\n"
"        [\"playlist-play-index\"] = {\"current\", \"none\"},\n"
"        [\"playlist-remove\"] = {\"current\"},\n"
"        [\"rescan-external-files\"] = {\"reselect\", \"keep-selection\"},\n"
"        [\"revert-seek\"] = {\"mark\", \"mark-permanent\"},\n"
"        [\"screenshot\"] = {\"subtitles\", \"video\", \"window\", \"each-frame\"},\n"
"        [\"stop\"] = {\"keep-playlist\"},\n"
"    }\n"
"    flags[\"playlist-prev\"] = flags[\"playlist-next\"]\n"
"    flags[\"screenshot-raw\"] = flags.screenshot\n"
"\n"
"    return flags[command]\n"
"end\n"
"\n"
"local function command_flags_at_2nd_argument_list(command)\n"
"    local flags = {\n"
"        [\"apply-profile\"] = {\"apply\", \"restore\"},\n"
"        [\"frame-step\"] = {\"play\", \"seek\", \"mute\"},\n"
"        [\"loadfile\"] = {\"replace\", \"append\", \"append-play\", \"insert-next\",\n"
"                        \"insert-next-play\", \"insert-at\", \"insert-at-play\"},\n"
"        [\"screenshot-to-file\"] = {\"subtitles\", \"video\", \"window\", \"each-frame\"},\n"
"        [\"screenshot-raw\"] = {\"bgr0\", \"bgra\", \"rgba\", \"rgba64\"},\n"
"        [\"seek\"] = {\"relative\", \"absolute\", \"absolute-percent\",\n"
"                    \"relative-percent\", \"keyframes\", \"exact\"},\n"
"        [\"sub-add\"] = {\"select\", \"auto\", \"cached\"},\n"
"        [\"sub-seek\"] = {\"primary\", \"secondary\"},\n"
"    }\n"
"    flags.loadlist = flags.loadfile\n"
"    flags[\"audio-add\"] = flags[\"sub-add\"]\n"
"    flags[\"video-add\"] = flags[\"sub-add\"]\n"
"    flags[\"sub-step\"] = flags[\"sub-seek\"]\n"
"\n"
"    return flags[command]\n"
"end\n"
"\n"
"local function handle_flags(command, arg_index, flags)\n"
"    for _, cmd in pairs(get_commands()) do\n"
"        if cmd.name == command then\n"
"            if cmd.args[arg_index] and cmd.args[arg_index].type == \"Flags\" then\n"
"                break\n"
"            else\n"
"                return\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    local plus_pos = flags:find(\"%+[^%+]*\044\")\n"
"\n"
"    if plus_pos then\n"
"        completion_pos = completion_pos + plus_pos\n"
"    end\n"
"end\n"
"\n"
"local function executable_list()\n"
"    local executable_map = {}\n"
"    local path = os.getenv(\"PATH\") or \"\"\n"
"    local separator = platform == \"windows\" and \";\" or \":\"\n"
"    local exts = {}\n"
"\n"
"    for ext in (os.getenv(\"PATHEXT\") or \"\"):gmatch(\"[^;]+\") do\n"
"        exts[ext:lower()] = true\n"
"    end\n"
"\n"
"    for directory in path:gmatch(\"[^\" .. separator .. \"]+\") do\n"
"        for _, executable in pairs(utils.readdir(directory, \"files\") or {}) do\n"
"            if not next(exts) or exts[(executable:match(\"%.%w+\044\") or \"\"):lower()] then\n"
"                executable_map[executable] = true\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    local executables = {}\n"
"    for executable, _ in pairs(executable_map) do\n"
"        executables[#executables + 1] = executable\n"
"    end\n"
"\n"
"    return executables\n"
"end\n"
"\n"
"local function filter_label_list(type)\n"
"    local values = {\"all\"}\n"
"\n"
"    for _, value in pairs(mp.get_property_native(type)) do\n"
"        if value.label then\n"
"            values[#values + 1] = value.label\n"
"        end\n"
"    end\n"
"\n"
"    return values\n"
"end\n"
"\n"
"local function complete(before_cur)\n"
"    local tokens = {}\n"
"    local first_useful_token_index = 1\n"
"    local completions\n"
"\n"
"    local begin_new_token = true\n"
"    local last_quote\n"
"    for pos, char in before_cur:gmatch(\"()(.)\") do\n"
"        if char:find(\"[%s;]\") and not last_quote then\n"
"            begin_new_token = true\n"
"            if char == \";\" then\n"
"                first_useful_token_index = #tokens + 1\n"
"            end\n"
"        elseif begin_new_token then\n"
"            tokens[#tokens + 1] = { text = char, pos = pos }\n"
"            last_quote = char:match(\"['\\\"]\")\n"
"            begin_new_token = false\n"
"        else\n"
"            tokens[#tokens].text = tokens[#tokens].text .. char\n"
"            if char == last_quote then\n"
"                last_quote = nil\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    completion_append = last_quote or \"\"\n"
"\n"
"    -- Strip quotes from tokens.\n"
"    for _, token in pairs(tokens) do\n"
"        if token.text:find('^\"') then\n"
"            token.text = token.text:sub(2):gsub('\"\044', \"\")\n"
"            token.pos = token.pos + 1\n"
"        elseif token.text:find(\"^'\") then\n"
"            token.text = token.text:sub(2):gsub('\"\044', \"\")\n"
"            token.pos = token.pos + 1\n"
"        end\n"
"    end\n"
"\n"
"    -- Skip command prefixes because it is not worth lumping them together with\n"
"    -- command completions when they are useless for interactive usage.\n"
"    local command_prefixes = {\n"
"        [\"osd-auto\"] = true, [\"no-osd\"] = true, [\"osd-bar\"] = true,\n"
"        [\"osd-msg\"] = true, [\"osd-msg-bar\"] = true, [\"raw\"] = true,\n"
"        [\"expand-properties\"] = true, [\"repeatable\"] = true,\n"
"        [\"nonrepeatable\"] = true, [\"nonscalable\"] = true,\n"
"        [\"async\"] = true, [\"sync\"] = true\n"
"    }\n"
"\n"
"    -- Add an empty token if the cursor is after whitespace or ; to simplify\n"
"    -- comparisons.\n"
"    if before_cur == \"\" or before_cur:find(\"[%s;]\044\") then\n"
"        tokens[#tokens + 1] = { text = \"\", pos = #before_cur + 1 }\n"
"    end\n"
"\n"
"    while tokens[first_useful_token_index] and\n"
"          command_prefixes[tokens[first_useful_token_index].text] do\n"
"        if first_useful_token_index == #tokens then\n"
"            return\n"
"        end\n"
"\n"
"        first_useful_token_index = first_useful_token_index + 1\n"
"    end\n"
"\n"
"    completion_pos = tokens[#tokens].pos\n"
"\n"
"    local add_actions = {\n"
"        [\"add\"] = true, [\"append\"] = true, [\"pre\"] = true, [\"set\"] = true\n"
"    }\n"
"\n"
"    local first_useful_token = tokens[first_useful_token_index]\n"
"\n"
"    local property_pos = before_cur:match(\"\044{[=>]\077()[%w_/-]*\044\")\n"
"    if property_pos then\n"
"        completion_pos = property_pos\n"
"        completions = property_list()\n"
"        completion_append = \"}\"\n"
"    elseif #tokens == first_useful_token_index then\n"
"        completions = command_list()\n"
"        completions[#completions + 1] = \"help\"\n"
"    elseif #tokens == first_useful_token_index + 1 then\n"
"        if first_useful_token.text == \"set\" or\n"
"           first_useful_token.text == \"add\" or\n"
"           first_useful_token.text == \"cycle\" or\n"
"           first_useful_token.text == \"cycle-values\" or\n"
"           first_useful_token.text == \"multiply\" then\n"
"            completions = property_list()\n"
"        elseif first_useful_token.text == \"help\" then\n"
"            completions = command_list()\n"
"        elseif first_useful_token.text == \"apply-profile\" then\n"
"            completions = profile_list()\n"
"        elseif first_useful_token.text == \"change-list\" then\n"
"            completions = list_option_list()\n"
"        elseif first_useful_token.text == \"run\" then\n"
"            completions = executable_list()\n"
"        elseif first_useful_token.text == \"vf\" or\n"
"               first_useful_token.text == \"af\" then\n"
"            completions = list_option_action_list(first_useful_token.text)\n"
"        elseif first_useful_token.text == \"vf-command\" or\n"
"               first_useful_token.text == \"af-command\" then\n"
"            completions = filter_label_list(first_useful_token.text:sub(1,2))\n"
"        elseif has_file_argument(first_useful_token.text) then\n"
"            completions = handle_file_completion(before_cur)\n"
"        else\n"
"            completions = command_flags_at_1st_argument_list(first_useful_token.text)\n"
"            handle_flags(first_useful_token.text, 1, tokens[#tokens].text)\n"
"        end\n"
"    elseif first_useful_token.text == \"cycle-values\" then\n"
"        completions = handle_choice_completion(tokens[first_useful_token_index + 1].text,\n"
"                                               before_cur)\n"
"    elseif first_useful_token.text == \"run\" then\n"
"        completions = handle_file_completion(before_cur)\n"
"    elseif #tokens == first_useful_token_index + 2 then\n"
"        if first_useful_token.text == \"set\" then\n"
"            completions = handle_choice_completion(tokens[first_useful_token_index + 1].text,\n"
"                                                   before_cur)\n"
"        elseif first_useful_token.text == \"change-list\" then\n"
"            completions = list_option_action_list(tokens[first_useful_token_index + 1].text)\n"
"        elseif first_useful_token.text == \"vf\" or\n"
"               first_useful_token.text == \"af\" then\n"
"            if add_actions[tokens[first_useful_token_index + 1].text] then\n"
"                completions = handle_choice_completion(first_useful_token.text, before_cur)\n"
"            elseif tokens[first_useful_token_index + 1].text == \"remove\" then\n"
"                completions = list_option_value_list(first_useful_token.text)\n"
"            end\n"
"        else\n"
"            completions = command_flags_at_2nd_argument_list(first_useful_token.text)\n"
"            handle_flags(first_useful_token.text, 2, tokens[#tokens].text)\n"
"        end\n"
"    elseif #tokens == first_useful_token_index + 3 then\n"
"        if first_useful_token.text == \"change-list\" then\n"
"            if add_actions[tokens[first_useful_token_index + 2].text] then\n"
"                completions = handle_choice_completion(tokens[first_useful_token_index + 1].text,\n"
"                                                       before_cur)\n"
"            elseif tokens[first_useful_token_index + 2].text == \"remove\" then\n"
"                completions = list_option_value_list(tokens[first_useful_token_index + 1].text)\n"
"            end\n"
"        elseif first_useful_token.text == \"dump-cache\" then\n"
"            completions = handle_file_completion(before_cur)\n"
"        end\n"
"    end\n"
"\n"
"    return completions or {}, completion_pos, completion_append\n"
"end\n"
"\n"
"local function open(text, cursor_position, keep_open)\n"
"    input.get({\n"
"        prompt = \">\",\n"
"        submit = submit,\n"
"        keep_open = keep_open,\n"
"        opened = opened,\n"
"        closed = closed,\n"
"        complete = complete,\n"
"        autoselect_completion = true,\n"
"        default_text = text,\n"
"        cursor_position = cursor_position,\n"
"        history_path = options.persist_history\n"
"                       and mp.command_native({\"expand-path\", options.history_path}) or nil,\n"
"    })\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"open\", function ()\n"
"    open(last_text, last_cursor_position, true)\n"
"end)\n"
"\n"
"-- Open the console with passed text and cursor_position arguments\n"
"mp.register_script_message(\"type\", open)\n"
"\n"
"mp.register_event(\"log-message\", function(e)\n"
"    -- Ignore log messages from the OSD because of paranoia, since writing them\n"
"    -- to the OSD could generate more messages in an infinite loop.\n"
"    if e.prefix:sub(1, 3) == \"osd\" then return end\n"
"\n"
"    -- Ignore messages output by this script.\n"
"    if e.prefix == mp.get_script_name() then return end\n"
"\n"
"    -- Ignore buffer overflow warning messages. Overflowed log messages would\n"
"    -- have been offscreen anyway.\n"
"    if e.prefix == \"overflow\" then return end\n"
"\n"
"    -- Filter out trace-level log messages, even if the terminal-default log\n"
"    -- level includes them. These aren\"t too useful for an on-screen display\n"
"    -- without scrollback and they include messages that are generated from the\n"
"    -- OSD display itself.\n"
"    if e.level == \"trace\" then return end\n"
"\n"
"    -- Avoid logging debug messages infinitely.\n"
"    if e.prefix == \"cplayer\" and\n"
"       e.text:find('^Run command: script%-message%-to, flags=64, args=%[target=\"console\"') then\n"
"        return\n"
"    end\n"
"\n"
"    -- Use color for debug/v/warn/error/fatal messages.\n"
"    input.log(\"[\" .. e.prefix .. \"] \" .. e.text:sub(1, -2), styles[e.level],\n"
"              terminal_styles[e.level])\n"
"end)\n"
"\n"
"-- Enable log messages. In silent mode, mpv will queue log messages in a buffer\n"
"-- until enable_messages is called again without the silent: prefix.\n"
"mp.enable_messages(\"silent:terminal-default\")\n"
"\n"
"require \"mp.options\".read_options(options)\n"
