// Generated from player/lua/console.lua

"-- Copyright (C) 2019 the mpv developers\n"
"--\n"
"-- Permission to use, copy, modify, and/or distribute this software for any\n"
"-- purpose with or without fee is hereby granted, provided that the above\n"
"-- copyright notice and this permission notice appear in all copies.\n"
"--\n"
"-- THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES\n"
"-- WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF\n"
"-- MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY\n"
"-- SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES\n"
"-- WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION\n"
"-- OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN\n"
"-- CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.\n"
"\n"
"local utils = require 'mp.utils'\n"
"local assdraw = require 'mp.assdraw'\n"
"\n"
"local function detect_platform()\n"
"    local platform = mp.get_property_native('platform')\n"
"    if platform == 'darwin' or platform == 'windows' then\n"
"        return platform\n"
"    elseif os.getenv('WAYLAND_DISPLAY') or os.getenv('WAYLAND_SOCKET') then\n"
"        return 'wayland'\n"
"    end\n"
"    return 'x11'\n"
"end\n"
"\n"
"local platform = detect_platform()\n"
"\n"
"-- Default options\n"
"local opts = {\n"
"    font = \"\",\n"
"    font_size = 24,\n"
"    border_size = 1.65,\n"
"    background_alpha = 80,\n"
"    padding = 10,\n"
"    menu_outline_size = 0,\n"
"    menu_outline_color = '#FFFFFF',\n"
"    corner_radius = 8,\n"
"    margin_x = -1,\n"
"    margin_y = -1,\n"
"    scale_with_window = \"auto\",\n"
"    selected_color = '#222222',\n"
"    selected_back_color = '#FFFFFF',\n"
"    match_color = '#0088FF',\n"
"    case_sensitive = platform ~= 'windows' and true or false,\n"
"    history_dedup = true,\n"
"    font_hw_ratio = 'auto',\n"
"}\n"
"\n"
"local styles = {\n"
"    error = '{\\\\1c&H7a77f2&}',\n"
"    completion = '{\\\\1c&Hcc99cc&}',\n"
"}\n"
"for key, style in pairs(styles) do\n"
"    styles[key] = style .. '{\\\\3c&H111111&}'\n"
"end\n"
"\n"
"local terminal_styles = {\n"
"    error = '\\027[31m',\n"
"    selected_completion = '\\027[7m',\n"
"    default_item = '\\027[1m',\n"
"    disabled = '\\027[38;5;8m',\n"
"    matched_position = '\\027[34m',\n"
"    match_end = '\\027[39m',\n"
"}\n"
"\n"
"local open = false\n"
"local osd_msg_active = false\n"
"local insert_mode = false\n"
"local pending_update = false\n"
"local ime_active = false\n"
"local line = ''\n"
"local cursor = 1\n"
"local prompt\n"
"local id\n"
"\n"
"local histories = {}\n"
"local history = {}\n"
"local history_pos = 1\n"
"local searching_history = false\n"
"local history_paths = {}\n"
"local histories_to_save = {}\n"
"\n"
"local log_buffers = {}\n"
"local key_bindings = {}\n"
"local dont_bind_up_down = false\n"
"local overlay = mp.create_osd_overlay('ass-events')\n"
"local width_overlay = mp.create_osd_overlay('ass-events')\n"
"local global_margins = { t = 0, b = 0 }\n"
"local input_caller\n"
"local keep_open = false\n"
"\n"
"local completion_buffer = {}\n"
"local selected_completion_index\n"
"local completion_pos\n"
"local completion_append\n"
"local completion_old_line\n"
"local completion_old_cursor\n"
"local autoselect_completion\n"
"local has_completions\n"
"\n"
"local selectable_items\n"
"local matches = {}\n"
"local selected_match = 1\n"
"local first_match_to_print = 1\n"
"local default_item\n"
"local item_positions = {}\n"
"local max_item_width = 0\n"
"\n"
"local complete\n"
"local cycle_through_completions\n"
"local set_active\n"
"\n"
"local property_cache = {}\n"
"\n"
"\n"
"local function get_property_cached(name, def)\n"
"    if property_cache[name] ~= nil then\n"
"        return property_cache[name]\n"
"    end\n"
"    return def\n"
"end\n"
"\n"
"local function get_font()\n"
"    if opts.font ~= '' then\n"
"        return opts.font\n"
"    end\n"
"\n"
"    if not has_completions then\n"
"        return\n"
"    end\n"
"\n"
"    -- Pick a better default font for Windows and macOS\n"
"    if platform == 'windows' then\n"
"        return 'Consolas'\n"
"    end\n"
"\n"
"    if platform == 'darwin' then\n"
"        return 'Menlo'\n"
"    end\n"
"\n"
"    return 'monospace'\n"
"end\n"
"\n"
"local function get_margin_x()\n"
"    return opts.margin_x > -1 and opts.margin_x or mp.get_property_native('osd-margin-x')\n"
"end\n"
"\n"
"local function get_margin_y()\n"
"    return opts.margin_y > -1 and opts.margin_y or mp.get_property_native('osd-margin-y')\n"
"end\n"
"\n"
"\n"
"-- Naive helper function to find the next UTF-8 character in 'str' after 'pos'\n"
"-- by skipping continuation bytes. Assumes 'str' contains valid UTF-8.\n"
"local function next_utf8(str, pos)\n"
"    if pos > str:len() then return pos end\n"
"    repeat\n"
"        pos = pos + 1\n"
"    until pos > str:len() or str:byte(pos) < 0x80 or str:byte(pos) > 0xbf\n"
"    return pos\n"
"end\n"
"\n"
"-- As above, but finds the previous UTF-8 character in 'str' before 'pos'\n"
"local function prev_utf8(str, pos)\n"
"    if pos <= 1 then return pos end\n"
"    repeat\n"
"        pos = pos - 1\n"
"    until pos <= 1 or str:byte(pos) < 0x80 or str:byte(pos) > 0xbf\n"
"    return pos\n"
"end\n"
"\n"
"local function len_utf8(str)\n"
"    local len = 0\n"
"    local pos = 1\n"
"    while pos <= str:len() do\n"
"        pos = next_utf8(str, pos)\n"
"        len = len + 1\n"
"    end\n"
"    return len\n"
"end\n"
"\n"
"local function utf8_positions(str)\n"
"    local pos = 1\n"
"    local positions = {true}\n"
"\n"
"    while pos <= str:len() do\n"
"        pos = next_utf8(str, pos)\n"
"        positions[pos] = true\n"
"    end\n"
"\n"
"    return positions\n"
"end\n"
"\n"
"\n"
"-- Functions to calculate the font width.\n"
"local width_length_ratio = 0.5\n"
"local osd_width, osd_height = 100, 100\n"
"local text_osd = mp.create_osd_overlay('ass-events')\n"
"text_osd.compute_bounds, text_osd.hidden = true, true\n"
"\n"
"local function measure_bounds(ass_text)\n"
"    text_osd.res_x, text_osd.res_y = osd_width, osd_height\n"
"    text_osd.data = ass_text\n"
"    local res = text_osd:update()\n"
"    return res.x0, res.y0, res.x1, res.y1\n"
"end\n"
"\n"
"---Measure text width and normalize to a font size of 1\n"
"---text has to be ass safe\n"
"local function normalized_text_width(text, size, horizontal)\n"
"    local align, rotation = horizontal and 7 or 1, horizontal and 0 or -90\n"
"    local template = '{\\\\pos(0,0)\\\\rDefault\\\\blur0\\\\bord0\\\\shad0\\\\q2\\\\an%s\\\\fs%s\\\\fn%s\\\\frz%s}%s'\n"
"    size = size / 0.8\n"
"    local width\n"
"    -- Limit to 5 iterations\n"
"    local repetitions_left = 5\n"
"    for i = 1, repetitions_left do\n"
"        size = size * 0.8\n"
"        local ass = assdraw.ass_new()\n"
"        ass.text = template:format(align, size, get_font(), rotation, text)\n"
"        local _, _, x1, y1 = measure_bounds(ass.text)\n"
"        -- Check if nothing got clipped\n"
"        if x1 and x1 < osd_width and y1 < osd_height then\n"
"            width = horizontal and x1 or y1\n"
"            break\n"
"        end\n"
"        if i == repetitions_left then\n"
"            width = 0\n"
"        end\n"
"    end\n"
"    return width / size, horizontal and osd_width or osd_height\n"
"end\n"
"\n"
"local function fit_on_osd(text)\n"
"    local estimated_width = #text * width_length_ratio\n"
"    if osd_width >= osd_height then\n"
"        -- Fill the osd as much as possible, bigger is more accurate.\n"
"        return math.min(osd_width / estimated_width, osd_height), true\n"
"    else\n"
"        return math.min(osd_height / estimated_width, osd_width), false\n"
"    end\n"
"end\n"
"\n"
"local measured_font_hw_ratio = nil\n"
"local function get_font_hw_ratio()\n"
"    local font_hw_ratio = tonumber(opts.font_hw_ratio)\n"
"    if font_hw_ratio then\n"
"        return font_hw_ratio\n"
"    end\n"
"    if not measured_font_hw_ratio then\n"
"        local alphabet = 'abcdefghijklmnopqrstuvwxyz'\n"
"        local text = alphabet:rep(3)\n"
"        local size, horizontal = fit_on_osd(text)\n"
"        local normalized_width = normalized_text_width(text, size * 0.9, horizontal)\n"
"        measured_font_hw_ratio = #text / normalized_width * 0.95\n"
"    end\n"
"    return measured_font_hw_ratio\n"
"end\n"
"\n"
"\n"
"-- Escape a string for verbatim display on the OSD\n"
"local function ass_escape(str)\n"
"    return mp.command_native({'escape-ass', str})\n"
"end\n"
"\n"
"local function should_scale()\n"
"    return opts.scale_with_window == \"yes\" or\n"
"           (opts.scale_with_window == \"auto\" and mp.get_property_native(\"osd-scale-by-window\"))\n"
"end\n"
"\n"
"local function scale_factor()\n"
"    if should_scale() and osd_height > 0 then\n"
"        return osd_height / 720\n"
"    end\n"
"\n"
"    return get_property_cached('display-hidpi-scale', 1)\n"
"end\n"
"\n"
"local function terminal_output()\n"
"    -- Unlike vo-configured, current-vo doesn't become falsy while switching VO,\n"
"    -- which would print the log to the OSD.\n"
"    return not mp.get_property('current-vo') or not mp.get_property_native('video-osd')\n"
"end\n"
"\n"
"local function get_scaled_osd_dimensions()\n"
"    local scale = scale_factor()\n"
"\n"
"    return osd_width / scale, osd_height / scale\n"
"end\n"
"\n"
"local function get_line_height()\n"
"    return selectable_items and opts.font_size * 1.1 or opts.font_size\n"
"end\n"
"\n"
"local function calculate_max_lines()\n"
"    if terminal_output() then\n"
"        -- Subtract 1 for the input line and for each line in the status line.\n"
"        -- This does not detect wrapped lines.\n"
"        return mp.get_property_native('term-size/h', 24) - 2 -\n"
"               select(2, mp.get_property('term-status-msg'):gsub('\\\\n', ''))\n"
"    end\n"
"\n"
"    return math.floor((select(2, get_scaled_osd_dimensions())\n"
"                       * (1 - global_margins.t - global_margins.b)\n"
"                       - get_margin_y() - (selectable_items and opts.padding * 2 or 0))\n"
"                      / get_line_height()\n"
"                      -- Subtract 1 for the input line and 0.5 for the empty\n"
"                      -- line between the log and the input line.\n"
"                      - 1.5)\n"
"end\n"
"\n"
"local function calculate_max_item_width()\n"
"    if not selectable_items or terminal_output() then\n"
"        return\n"
"    end\n"
"\n"
"    local longest_item = prompt .. ('a'):rep(9)\n"
"    for _, item in pairs(selectable_items) do\n"
"        if #item > #longest_item then\n"
"            longest_item = item\n"
"        end\n"
"    end\n"
"\n"
"    local osd_w, osd_h = get_scaled_osd_dimensions()\n"
"    local font = get_font()\n"
"    width_overlay.compute_bounds = true\n"
"    width_overlay.hidden = true\n"
"    width_overlay.res_x = osd_w\n"
"    width_overlay.res_y = osd_h\n"
"    width_overlay.data = '{\\\\fs' .. opts.font_size ..\n"
"                         (font and '\\\\fn' .. font or '') .. '\\\\q2}' ..\n"
"                         ass_escape(longest_item)\n"
"    local result = width_overlay:update()\n"
"    max_item_width = math.min(result.x1 - result.x0,\n"
"                              osd_w - get_margin_x() * 2 - opts.padding * 2)\n"
"end\n"
"\n"
"local function should_highlight_completion(i)\n"
"    return i == selected_completion_index or\n"
"           (i == 1 and selected_completion_index == 0 and autoselect_completion)\n"
"end\n"
"\n"
"local function mpv_color_to_ass(color)\n"
"    return color:sub(8,9) .. color:sub(6,7) ..  color:sub(4,5),\n"
"           string.format('%x', 255 - tonumber('0x' .. color:sub(2,3)))\n"
"end\n"
"\n"
"local function option_color_to_ass(color)\n"
"    return color:sub(6,7) .. color:sub(4,5) ..  color:sub(2,3)\n"
"end\n"
"\n"
"local function get_selected_ass()\n"
"    local color, alpha = mpv_color_to_ass(mp.get_property('osd-selected-color'))\n"
"    local outline_color, outline_alpha =\n"
"        mpv_color_to_ass(mp.get_property('osd-selected-outline-color'))\n"
"    return '{\\\\b1\\\\1c&H' .. color .. '&\\\\1a&H' .. alpha ..\n"
"           '&\\\\3c&H' .. outline_color .. '&\\\\3a&H' .. outline_alpha .. '&}'\n"
"end\n"
"\n"
"-- Takes a list of strings, a max width in characters and\n"
"-- optionally a max row count.\n"
"-- The result contains at least one column.\n"
"-- Rows are cut off from the top if rows_max is specified.\n"
"-- returns a string containing the formatted table and the row count\n"
"local function format_grid(list, width_max, rows_max)\n"
"    if #list == 0 then\n"
"        return '', 0\n"
"    end\n"
"\n"
"    local spaces_min = 2\n"
"    local spaces_max = 8\n"
"    local list_size = #list\n"
"    local column_count = 1\n"
"    local row_count = list_size\n"
"    local column_widths\n"
"    -- total width without spacing\n"
"    local width_total = 0\n"
"\n"
"    local list_widths = {}\n"
"    for i, item in ipairs(list) do\n"
"        list_widths[i] = len_utf8(item)\n"
"    end\n"
"\n"
"    -- use as many columns as possible\n"
"    for columns = 2, list_size do\n"
"        local rows_lower_bound = math.min(rows_max, math.ceil(list_size / columns))\n"
"        local rows_upper_bound = math.min(rows_max, list_size,\n"
"                                          math.ceil(list_size / (columns - 1) - 1))\n"
"        for rows = rows_upper_bound, rows_lower_bound, -1 do\n"
"            local cw = {}\n"
"            width_total = 0\n"
"\n"
"            -- find out width of each column\n"
"            for column = 1, columns do\n"
"                local width = 0\n"
"                for row = 1, rows do\n"
"                    local i = row + (column - 1) * rows\n"
"                    local item_width = list_widths[i]\n"
"                    if not item_width then break end\n"
"                    if width < item_width then\n"
"                        width = item_width\n"
"                    end\n"
"                end\n"
"                cw[column] = width\n"
"                width_total = width_total + width\n"
"                if width_total + (columns - 1) * spaces_min > width_max then\n"
"                    break\n"
"                end\n"
"            end\n"
"\n"
"            if width_total + (columns - 1) * spaces_min <= width_max then\n"
"                row_count = rows\n"
"                column_count = columns\n"
"                column_widths = cw\n"
"            else\n"
"                break\n"
"            end\n"
"        end\n"
"        if width_total + (columns - 1) * spaces_min > width_max then\n"
"            break\n"
"        end\n"
"    end\n"
"\n"
"    local spaces = math.floor((width_max - width_total) / (column_count - 1))\n"
"    spaces = math.max(spaces_min, math.min(spaces_max, spaces))\n"
"    local spacing = column_count > 1\n"
"                    and ass_escape(string.format('%' .. spaces .. 's', ' '))\n"
"                    or ''\n"
"\n"
"    local rows = {}\n"
"    for row = 1, row_count do\n"
"        local columns = {}\n"
"        for column = 1, column_count do\n"
"            local i = row + (column - 1) * row_count\n"
"            if i > #list then break end\n"
"            -- more then 99 leads to 'invalid format (width or precision too long)'\n"
"            local format_string = column == column_count and '%s'\n"
"                                  or '%-' .. math.min(column_widths[column], 99) .. 's'\n"
"            columns[column] = ass_escape(string.format(format_string, list[i]))\n"
"\n"
"            if should_highlight_completion(i) then\n"
"                columns[column] = get_selected_ass() .. columns[column] ..\n"
"                                  '{\\\\b\\\\1a&\\\\3a&}' .. styles.completion\n"
"            end\n"
"        end\n"
"        -- first row is at the bottom\n"
"        rows[row_count - row + 1] = table.concat(columns, spacing)\n"
"    end\n"
"    return table.concat(rows, ass_escape('\\n')), row_count\n"
"end\n"
"\n"
"local function fuzzy_find(needle, haystacks, case_sensitive)\n"
"    local result = require 'mp.fzy'.filter(needle, haystacks, case_sensitive)\n"
"    table.sort(result, function (i, j)\n"
"        if i[3] ~= j[3] then\n"
"            return i[3] > j[3]\n"
"        end\n"
"\n"
"        return i[1] < j[1]\n"
"    end)\n"
"\n"
"    return result\n"
"end\n"
"\n"
"local function get_matches_to_print(terminal)\n"
"    if not selectable_items or selected_match == 0 then\n"
"        return {}\n"
"    end\n"
"\n"
"    local items = {}\n"
"    local max_lines = calculate_max_lines()\n"
"    local escape = terminal and function (str) return str end or ass_escape\n"
"    local highlight = terminal and terminal_styles.matched_position or\n"
"                     '{\\\\1c&H' .. option_color_to_ass(opts.match_color) .. '}'\n"
"\n"
"    if selected_match < first_match_to_print then\n"
"        first_match_to_print = selected_match\n"
"    elseif selected_match > first_match_to_print + max_lines - 1 then\n"
"        first_match_to_print = selected_match - max_lines + 1\n"
"    end\n"
"\n"
"    local last_match_to_print  = math.min(first_match_to_print + max_lines - 1,\n"
"                                          #matches)\n"
"\n"
"    for i = first_match_to_print, last_match_to_print do\n"
"        local item = ''\n"
"        local end_highlight = terminal and terminal_styles.match_end or '{\\\\1c}'\n"
"\n"
"        if terminal then\n"
"            if matches[i].index == default_item then\n"
"                item = terminal_styles.default_item\n"
"            end\n"
"            if i == selected_match then\n"
"                item = item .. terminal_styles.selected_completion\n"
"            end\n"
"        else\n"
"            if i == selected_match then\n"
"                if searching_history and\n"
"                   mp.get_property('osd-border-style') == 'outline-and-shadow' then\n"
"                    item = get_selected_ass()\n"
"                else\n"
"                    item = '{\\\\1c&H' .. option_color_to_ass(opts.selected_color) .. '&}'\n"
"                end\n"
"                end_highlight = item\n"
"            end\n"
"        end\n"
"\n"
"        local char_positions = utf8_positions(matches[i].text)\n"
"        local start_of_last_match = matches[i].positions[1]\n"
"\n"
"        if not start_of_last_match then\n"
"            item = item .. escape(matches[i].text)\n"
"        elseif start_of_last_match > 1 then\n"
"            item = item .. escape(matches[i].text:sub(1, start_of_last_match - 1))\n"
"        end\n"
"\n"
"        for j, pos in ipairs(matches[i].positions) do\n"
"            local last_pos = matches[i].positions[j - 1]\n"
"\n"
"            if last_pos and pos > last_pos + 1 then\n"
"                if char_positions[start_of_last_match] and char_positions[last_pos + 1] then\n"
"                    item = item .. highlight ..\n"
"                           escape(matches[i].text:sub(start_of_last_match, last_pos)) ..\n"
"                           end_highlight ..\n"
"                           escape(matches[i].text:sub(last_pos + 1, pos - 1))\n"
"                else\n"
"                    item = item .. escape(matches[i].text:sub(start_of_last_match, pos - 1))\n"
"                end\n"
"\n"
"                start_of_last_match = pos\n"
"            end\n"
"        end\n"
"\n"
"        if start_of_last_match then\n"
"            local last_pos = matches[i].positions[#matches[i].positions]\n"
"\n"
"            if char_positions[start_of_last_match] and char_positions[last_pos + 1] then\n"
"                item = item .. highlight ..\n"
"                       escape(matches[i].text:sub(start_of_last_match, last_pos)) ..\n"
"                       end_highlight ..\n"
"                       escape(matches[i].text:sub(last_pos + 1))\n"
"            else\n"
"                item = item .. escape(matches[i].text:sub(start_of_last_match))\n"
"            end\n"
"        end\n"
"\n"
"        items[#items + 1] = item\n"
"    end\n"
"\n"
"    return items\n"
"end\n"
"\n"
"local function update_overlay(data, res_x, res_y, z)\n"
"    if overlay.data == data and\n"
"       overlay.res_x == res_x and\n"
"       overlay.res_y == res_y and\n"
"       overlay.z == z then\n"
"        return\n"
"    end\n"
"\n"
"    overlay.data = data\n"
"    overlay.res_x = res_x\n"
"    overlay.res_y = res_y\n"
"    overlay.z = z\n"
"    overlay:update()\n"
"end\n"
"\n"
"local function print_to_terminal()\n"
"    -- Clear the log after closing the console.\n"
"    if not open then\n"
"        if osd_msg_active then\n"
"            mp.osd_message('')\n"
"        end\n"
"        osd_msg_active = false\n"
"        return\n"
"    end\n"
"\n"
"    local log = ''\n"
"    local counter = ''\n"
"    if selectable_items then\n"
"        if #selectable_items > calculate_max_lines() then\n"
"            local digits = math.ceil(math.log(#selectable_items, 10))\n"
"            counter = terminal_styles.disabled ..\n"
"                      '[' .. string.format('%0' .. digits .. 'd', selected_match) ..\n"
"                      '/' .. string.format('%0' .. digits .. 'd', #matches) ..\n"
"                      ']\\027[0m '\n"
"        end\n"
"\n"
"        local clip = mp.get_property('term-clip-cc')\n"
"        for _, item in ipairs(get_matches_to_print(true)) do\n"
"            log = log .. clip .. item .. '\\027[0m\\n'\n"
"        end\n"
"    else\n"
"        for _, log_line in ipairs(log_buffers[id]) do\n"
"            log = log .. log_line.terminal_style .. log_line.text .. '\\027[0m\\n'\n"
"        end\n"
"\n"
"        for i, completion in ipairs(completion_buffer) do\n"
"            if should_highlight_completion(i) then\n"
"                log = log .. terminal_styles.selected_completion ..\n"
"                      completion .. '\\027[0m'\n"
"            else\n"
"                log = log .. completion\n"
"            end\n"
"            log = log .. (i < #completion_buffer and '\\t' or '\\n')\n"
"        end\n"
"    end\n"
"\n"
"    local before_cur = line:sub(1, cursor - 1)\n"
"    local after_cur = line:sub(cursor)\n"
"    -- Ensure there is a character with inverted colors to print.\n"
"    if after_cur == '' then\n"
"        after_cur = ' '\n"
"    end\n"
"\n"
"    mp.osd_message(log .. counter .. prompt .. ' ' .. before_cur .. '\\027[7m' ..\n"
"                   after_cur:sub(1, 1) .. '\\027[0m' .. after_cur:sub(2), 999)\n"
"    osd_msg_active = true\n"
"end\n"
"\n"
"local function render()\n"
"    pending_update = false\n"
"\n"
"    if terminal_output() then\n"
"        print_to_terminal()\n"
"        return\n"
"    end\n"
"\n"
"    -- Clear the OSD if the console was being printed to the terminal\n"
"    if osd_msg_active then\n"
"        mp.osd_message('')\n"
"        osd_msg_active = false\n"
"    end\n"
"\n"
"    -- Clear the OSD after closing the console\n"
"    if not open then\n"
"        update_overlay('', 0, 0, 0)\n"
"        return\n"
"    end\n"
"\n"
"    local ass = assdraw.ass_new()\n"
"    local osd_w, osd_h = get_scaled_osd_dimensions()\n"
"    local line_height = get_line_height()\n"
"    local max_lines = calculate_max_lines()\n"
"\n"
"    local x, y, alignment, clipping_coordinates\n"
"    if selectable_items and not searching_history then\n"
"        x = (osd_w - max_item_width) / 2\n"
"        y = osd_h / 2 - (math.min(#selectable_items, max_lines) + 1.5) * line_height / 2\n"
"        alignment = 7\n"
"        clipping_coordinates = '0,0,' .. x + max_item_width .. ',' .. osd_h\n"
"    else\n"
"        x = get_margin_x()\n"
"        y = osd_h * (1 - global_margins.b) - get_margin_y()\n"
"        alignment = 1\n"
"        -- Avoid drawing below topbar OSC when there are wrapped lines.\n"
"        local coordinate_top = math.floor(global_margins.t * osd_h + 0.5)\n"
"        clipping_coordinates = '0,' .. coordinate_top .. ',' .. osd_w .. ',' .. osd_h\n"
"    end\n"
"\n"
"    local font = get_font()\n"
"    -- Use the same blur value as the rest of the OSD. 288 is the OSD's\n"
"    -- PlayResY.\n"
"    local blur = mp.get_property_native('osd-blur') * osd_h / 288\n"
"    local border_style = mp.get_property('osd-border-style')\n"
"\n"
"    local style = '{\\\\r' ..\n"
"                  (font and '\\\\fn' .. font or '') ..\n"
"                  '\\\\fs' .. opts.font_size ..\n"
"                  '\\\\bord' .. opts.border_size .. '\\\\fsp0' ..\n"
"                  '\\\\blur' .. blur ..\n"
"                  (selectable_items and '\\\\q2' or '\\\\q1') ..\n"
"                  '\\\\clip(' .. clipping_coordinates .. ')}'\n"
"\n"
"    -- Create the cursor glyph as an ASS drawing. ASS will draw the cursor\n"
"    -- inline with the surrounding text, but it sets the advance to the width\n"
"    -- of the drawing. So the cursor doesn't affect layout too much, make it as\n"
"    -- thin as possible and make it appear to be 1px wide by giving it 0.5px\n"
"    -- horizontal borders.\n"
"    local color, alpha = mpv_color_to_ass(mp.get_property('osd-color'))\n"
"    local cheight = opts.font_size * 8\n"
"    local cglyph = '{\\\\r\\\\blur0' ..\n"
"                   (get_property_cached('focused') == false\n"
"                    and '\\\\alpha&HFF&' or '\\\\3a&H' .. alpha .. '&') ..\n"
"                   '\\\\3c&H' .. color .. '&' ..\n"
"                   '\\\\xbord0.5\\\\ybord0\\\\xshad0\\\\yshad1\\\\p4\\\\pbo24}' ..\n"
"                   'm 0 0 l 1 0 l 1 ' .. cheight .. ' l 0 ' .. cheight ..\n"
"                   '{\\\\p0}'\n"
"    local before_cur = ass_escape(line:sub(1, cursor - 1))\n"
"    local after_cur = ass_escape(line:sub(cursor))\n"
"\n"
"    local log_ass = ''\n"
"    local log_buffer = log_buffers[id] or {}\n"
"    for i = #log_buffer - math.min(max_lines, #log_buffer) + 1, #log_buffer do\n"
"        log_ass = log_ass .. style .. log_buffer[i].style ..\n"
"                  ass_escape(log_buffer[i].text) .. '\\\\N'\n"
"    end\n"
"\n"
"    local completion_ass = ''\n"
"    if completion_buffer[1] and not selectable_items then\n"
"        -- Estimate how many characters fit in one line\n"
"        -- Even with bottom-left anchoring,\n"
"        -- libass/ass_render.c:ass_render_event() subtracts --osd-margin-x from\n"
"        -- the maximum text width twice.\n"
"        -- TODO: --osd-margin-x should scale with osd-width and PlayResX to make\n"
"        -- the calculation accurate.\n"
"        local width_max = math.floor(\n"
"            (osd_w - x - mp.get_property_native('osd-margin-x') * 2)\n"
"            / opts.font_size * get_font_hw_ratio())\n"
"\n"
"        local completions, rows = format_grid(completion_buffer, width_max, max_lines)\n"
"        max_lines = max_lines - rows\n"
"        completion_ass = style .. styles.completion .. completions .. '\\\\N'\n"
"    end\n"
"\n"
"    -- Background\n"
"    if selectable_items and\n"
"       (not searching_history or border_style == 'background-box') then\n"
"        style = style .. '{\\\\bord0\\\\blur0\\\\4a&Hff&}'\n"
"        local back_color, back_alpha = mpv_color_to_ass(mp.get_property(\n"
"            border_style == 'background-box' and 'osd-back-color' or 'osd-outline-color'))\n"
"        if not searching_history then\n"
"            back_alpha = string.format('%x', opts.background_alpha)\n"
"        end\n"
"\n"
"        ass:new_event()\n"
"        ass:an(alignment)\n"
"        ass:pos(x, y)\n"
"        ass:append('{\\\\1c&H' .. back_color .. '&\\\\1a&H' .. back_alpha ..\n"
"                   '&\\\\bord' .. opts.menu_outline_size .. '\\\\3c&H' ..\n"
"                   option_color_to_ass(opts.menu_outline_color) .. '&}')\n"
"        if border_style == 'background-box' then\n"
"            ass:append('{\\\\4a&Hff&}')\n"
"        end\n"
"        ass:draw_start()\n"
"        ass:round_rect_cw(-opts.padding,\n"
"                          opts.padding * (alignment == 7 and -1 or 1),\n"
"                          max_item_width + opts.padding,\n"
"                          (1.5 + math.min(#matches, max_lines)) * line_height +\n"
"                          opts.padding * (alignment == 7 and 1 or 2),\n"
"                          opts.corner_radius, opts.corner_radius)\n"
"        ass:draw_stop()\n"
"    end\n"
"\n"
"    local items = get_matches_to_print()\n"
"    item_positions = {}\n"
"    for i, item in ipairs(items) do\n"
"        local item_y = alignment == 7\n"
"            and y + (1 + i) * line_height\n"
"            or y - (1.5 + #items - i) * line_height\n"
"\n"
"        if (first_match_to_print - 1 + i == selected_match or\n"
"            matches[first_match_to_print - 1 + i].index == default_item)\n"
"           and (not searching_history or border_style == 'background-box') then\n"
"            ass:new_event()\n"
"            ass:an(4)\n"
"            ass:pos(x, item_y)\n"
"            ass:append('{\\\\blur0\\\\bord0\\\\4aH&ff&\\\\1c&H' ..\n"
"                       option_color_to_ass(opts.selected_back_color) .. '&}')\n"
"            if first_match_to_print - 1 + i ~= selected_match then\n"
"                ass:append('{\\\\1aH&cc&}')\n"
"            end\n"
"            ass:draw_start()\n"
"            ass:rect_cw(-opts.padding, 0, max_item_width + opts.padding, line_height)\n"
"            ass:draw_stop()\n"
"        end\n"
"\n"
"        ass:new_event()\n"
"        ass:an(4)\n"
"        ass:pos(x, item_y)\n"
"        ass:append(style .. item)\n"
"\n"
"        item_positions[#item_positions + 1] =\n"
"            { item_y - line_height / 2, item_y + line_height / 2 }\n"
"    end\n"
"\n"
"    -- Scrollbar\n"
"    if selectable_items and #matches > max_lines then\n"
"        ass:new_event()\n"
"        ass:an(alignment + 2)\n"
"        ass:pos(x + max_item_width, y)\n"
"        ass:append(style)\n"
"        if not searching_history or border_style == 'background-box' then\n"
"            ass:append('{\\\\bord0\\\\4a&Hff&\\\\blur0}')\n"
"        end\n"
"        ass:append(selected_match .. '/' .. #matches)\n"
"\n"
"        local start_percentage = (first_match_to_print - 1) / #matches\n"
"        local end_percentage = (first_match_to_print - 1 + max_lines) / #matches\n"
"        if end_percentage - start_percentage < 0.04 then\n"
"            local diff = 0.04 - (end_percentage - start_percentage)\n"
"            start_percentage = start_percentage * (1 - diff)\n"
"            end_percentage = end_percentage + diff * (1 - end_percentage)\n"
"        end\n"
"\n"
"        local max_height = max_lines * line_height\n"
"        local bar_y = alignment == 7\n"
"                      and y + 1.5 * line_height + start_percentage * max_height\n"
"                      or y - 1.5 * line_height - max_height * (1 - end_percentage)\n"
"       local height = max_height * (end_percentage - start_percentage)\n"
"\n"
"        ass:new_event()\n"
"        ass:an(alignment)\n"
"        ass:append('{\\\\blur0\\\\4a&Hff&\\\\bord1}')\n"
"        ass:pos(x + max_item_width + opts.padding - 1, bar_y)\n"
"        ass:draw_start()\n"
"        ass:rect_cw(0, 0, -opts.padding / 2, height)\n"
"        ass:draw_stop()\n"
"    end\n"
"\n"
"    ass:new_event()\n"
"    ass:an(alignment)\n"
"    ass:pos(x, y)\n"
"    if not selectable_items then\n"
"        ass:append(log_ass .. '\\\\N' .. completion_ass)\n"
"    end\n"
"    ass:append(style .. ass_escape(prompt) .. ' ' .. before_cur)\n"
"    ass:append(cglyph)\n"
"    ass:append(style .. after_cur)\n"
"\n"
"    -- Redraw the cursor with the input text invisible. This will make the\n"
"    -- cursor appear in front of the text.\n"
"    ass:new_event()\n"
"    ass:an(alignment)\n"
"    ass:pos(x, y)\n"
"    ass:append(style .. '{\\\\alpha&HFF&}' .. ass_escape(prompt) .. ' ' .. before_cur)\n"
"    ass:append(cglyph)\n"
"    ass:append(style .. '{\\\\alpha&HFF&}' .. after_cur)\n"
"\n"
"    -- z with selectable_items needs to be greater than the OSC's.\n"
"    update_overlay(ass.text, osd_w, osd_h, selectable_items and 2000 or 0)\n"
"end\n"
"\n"
"local update_timer = nil\n"
"update_timer = mp.add_periodic_timer(0.05, function()\n"
"    if pending_update then\n"
"        render()\n"
"    else\n"
"        update_timer:kill()\n"
"    end\n"
"end)\n"
"update_timer:kill()\n"
"\n"
"-- Add a line to the history and deduplicate\n"
"local function history_add(text)\n"
"    if history[#history] == text or text == '' then\n"
"        return\n"
"    end\n"
"\n"
"    if opts.history_dedup then\n"
"        -- More recent entries are more likely to be repeated\n"
"        for i = #history, 1, -1 do\n"
"            if history[i] == text then\n"
"                table.remove(history, i)\n"
"                break\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    history[#history + 1] = text\n"
"\n"
"    if history_paths[id] then\n"
"        histories_to_save[id] =  histories_to_save[id] .. text .. '\\n'\n"
"    end\n"
"end\n"
"\n"
"local function handle_cursor_move()\n"
"    -- Don't show completions after a command is entered because they move its\n"
"    -- output up, and allow clearing completions by emptying the line.\n"
"    if line == '' then\n"
"        completion_buffer = {}\n"
"        render()\n"
"    else\n"
"        complete()\n"
"    end\n"
"end\n"
"\n"
"local function handle_edit()\n"
"    if not selectable_items then\n"
"        handle_cursor_move()\n"
"        mp.commandv('script-message-to', input_caller, 'input-event', 'edited',\n"
"                    utils.format_json({line}))\n"
"        return\n"
"    end\n"
"\n"
"    matches = {}\n"
"    for i, match in ipairs(fuzzy_find(line, selectable_items)) do\n"
"        matches[i] = {\n"
"            index = match[1],\n"
"            text = selectable_items[match[1]],\n"
"            positions = match[2],\n"
"        }\n"
"    end\n"
"\n"
"    if line == '' and default_item then\n"
"        selected_match = default_item\n"
"\n"
"        local max_lines = calculate_max_lines()\n"
"        first_match_to_print = math.max(1, selected_match + 1 - math.ceil(max_lines / 2))\n"
"        if first_match_to_print > #selectable_items - max_lines + 1 then\n"
"            first_match_to_print = math.max(1, #selectable_items - max_lines + 1)\n"
"        end\n"
"    else\n"
"        selected_match = 1\n"
"    end\n"
"\n"
"    render()\n"
"end\n"
"\n"
"-- Insert a character at the current cursor position (any_unicode)\n"
"local function handle_char_input(c)\n"
"    if insert_mode then\n"
"        line = line:sub(1, cursor - 1) .. c .. line:sub(next_utf8(line, cursor))\n"
"    else\n"
"        line = line:sub(1, cursor - 1) .. c .. line:sub(cursor)\n"
"    end\n"
"    cursor = cursor + #c\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Remove the character behind the cursor (Backspace)\n"
"local function handle_backspace()\n"
"    if cursor <= 1 then return end\n"
"    local prev = prev_utf8(line, cursor)\n"
"    line = line:sub(1, prev - 1) .. line:sub(cursor)\n"
"    cursor = prev\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Remove the character in front of the cursor (Del)\n"
"local function handle_del()\n"
"    if cursor > line:len() then return end\n"
"    line = line:sub(1, cursor - 1) .. line:sub(next_utf8(line, cursor))\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Toggle insert mode (Ins)\n"
"local function handle_ins()\n"
"    insert_mode = not insert_mode\n"
"end\n"
"\n"
"-- Move the cursor to the next character (Right)\n"
"local function next_char()\n"
"    cursor = next_utf8(line, cursor)\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Move the cursor to the previous character (Left)\n"
"local function prev_char()\n"
"    cursor = prev_utf8(line, cursor)\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Clear the current line (Ctrl+C)\n"
"local function clear()\n"
"    line = ''\n"
"    cursor = 1\n"
"    insert_mode = false\n"
"    history_pos = #history + 1\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Close the console if the current line is empty, otherwise delete the next\n"
"-- character (Ctrl+D)\n"
"local function maybe_exit()\n"
"    if line == '' then\n"
"        set_active(false)\n"
"    else\n"
"        handle_del()\n"
"    end\n"
"end\n"
"\n"
"local function unbind_mouse()\n"
"    mp.remove_key_binding('_console_mouse_move')\n"
"    mp.remove_key_binding('_console_mbtn_left')\n"
"end\n"
"\n"
"-- Run the current command or select the current item\n"
"local function submit()\n"
"    if searching_history then\n"
"        searching_history = false\n"
"        selectable_items = nil\n"
"        line = #matches > 0 and matches[selected_match].text or ''\n"
"        cursor = #line + 1\n"
"        handle_edit()\n"
"        unbind_mouse()\n"
"        return\n"
"    end\n"
"\n"
"    if selectable_items then\n"
"        if #matches > 0 then\n"
"            mp.commandv('script-message-to', input_caller, 'input-event', 'submit',\n"
"                        utils.format_json({matches[selected_match].index}))\n"
"        end\n"
"    else\n"
"        if selected_completion_index == 0 and autoselect_completion then\n"
"            cycle_through_completions()\n"
"        end\n"
"\n"
"        mp.commandv('script-message-to', input_caller, 'input-event', 'submit',\n"
"                    utils.format_json({line}))\n"
"\n"
"        history_add(line)\n"
"    end\n"
"\n"
"    if not keep_open then\n"
"        set_active(false)\n"
"    elseif not selectable_items then\n"
"        clear()\n"
"    end\n"
"end\n"
"\n"
"local function determine_hovered_item()\n"
"    local osd_w, _ = get_scaled_osd_dimensions()\n"
"    local scale = scale_factor()\n"
"    local mouse_pos = mp.get_property_native('mouse-pos')\n"
"    local mouse_x = mouse_pos.x / scale\n"
"    local mouse_y = mouse_pos.y / scale\n"
"    local item_x0 = (searching_history and get_margin_x() or (osd_w - max_item_width) / 2)\n"
"                    - opts.padding\n"
"\n"
"    if mouse_x < item_x0 or mouse_x > item_x0 + max_item_width + opts.padding * 2 then\n"
"        return\n"
"    end\n"
"\n"
"    for i, positions in ipairs(item_positions) do\n"
"        if mouse_y >= positions[1] and mouse_y <= positions[2] then\n"
"            return first_match_to_print - 1 + i\n"
"        end\n"
"    end\n"
"end\n"
"\n"
"local function bind_mouse()\n"
"    mp.add_forced_key_binding('MOUSE_MOVE', '_console_mouse_move', function()\n"
"        local item = determine_hovered_item()\n"
"        if item and item ~= selected_match then\n"
"            selected_match = item\n"
"            render()\n"
"        end\n"
"    end)\n"
"\n"
"    mp.add_forced_key_binding('MBTN_LEFT', '_console_mbtn_left', function()\n"
"        local item = determine_hovered_item()\n"
"        if item then\n"
"            selected_match = item\n"
"            submit()\n"
"        else\n"
"            set_active(false)\n"
"        end\n"
"    end)\n"
"end\n"
"\n"
"-- Go to the specified position in the command history\n"
"local function go_history(new_pos)\n"
"    local old_pos = history_pos\n"
"    history_pos = new_pos\n"
"\n"
"    -- Restrict the position to a legal value\n"
"    if history_pos > #history + 1 then\n"
"        history_pos = #history + 1\n"
"    elseif history_pos < 1 then\n"
"        history_pos = 1\n"
"    end\n"
"\n"
"    -- Do nothing if the history position didn't actually change\n"
"    if history_pos == old_pos then\n"
"        return\n"
"    end\n"
"\n"
"    -- If the user was editing a non-history line, save it as the last history\n"
"    -- entry. This makes it much less frustrating to accidentally hit Up/Down\n"
"    -- while editing a line.\n"
"    if old_pos == #history + 1 then\n"
"        history_add(line)\n"
"    end\n"
"\n"
"    -- Now show the history line (or a blank line for #history + 1)\n"
"    if history_pos <= #history then\n"
"        line = history[history_pos]\n"
"    else\n"
"        line = ''\n"
"    end\n"
"    cursor = line:len() + 1\n"
"    insert_mode = false\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Go to the specified relative position in the command history (Up, Down)\n"
"local function move_history(amount, is_wheel)\n"
"    if not selectable_items then\n"
"        go_history(history_pos + amount)\n"
"        return\n"
"    end\n"
"\n"
"    if is_wheel then\n"
"        local max_lines = calculate_max_lines()\n"
"\n"
"        -- Update selected_match only if it's the first or last printed item and\n"
"        -- there are hidden items.\n"
"        if (amount > 0 and selected_match == first_match_to_print\n"
"            and first_match_to_print - 1 + max_lines < #matches)\n"
"           or (amount < 0 and selected_match == first_match_to_print - 1 + max_lines\n"
"               and first_match_to_print > 1) then\n"
"            selected_match = selected_match + amount\n"
"        end\n"
"\n"
"        if amount > 0 and first_match_to_print < #matches - max_lines + 1\n"
"           or amount < 0 and first_match_to_print > 1 then\n"
"           -- math.min and math.max would only be needed with amounts other than\n"
"           -- 1 and -1.\n"
"            first_match_to_print = math.min(\n"
"                math.max(first_match_to_print + amount, 1), #matches - max_lines + 1)\n"
"        end\n"
"\n"
"        local item = determine_hovered_item()\n"
"        if item then\n"
"            selected_match = item\n"
"        end\n"
"\n"
"        render()\n"
"        return\n"
"    end\n"
"\n"
"    selected_match = selected_match + amount\n"
"    if selected_match > #matches then\n"
"        selected_match = 1\n"
"    elseif selected_match < 1 then\n"
"        selected_match = #matches\n"
"    end\n"
"    render()\n"
"end\n"
"\n"
"-- Go to the first command in the command history (PgUp)\n"
"local function handle_pgup()\n"
"    if selectable_items then\n"
"        selected_match = math.max(selected_match - calculate_max_lines() + 1, 1)\n"
"        render()\n"
"        return\n"
"    end\n"
"\n"
"    go_history(1)\n"
"end\n"
"\n"
"-- Stop browsing history and start editing a blank line (PgDown)\n"
"local function handle_pgdown()\n"
"    if selectable_items then\n"
"        selected_match = math.min(selected_match + calculate_max_lines() - 1, #matches)\n"
"        render()\n"
"        return\n"
"    end\n"
"\n"
"    go_history(#history + 1)\n"
"end\n"
"\n"
"local function search_history()\n"
"    if selectable_items or #history == 0 then\n"
"        return\n"
"    end\n"
"\n"
"    searching_history = true\n"
"    selectable_items = {}\n"
"\n"
"    for i = 1, #history do\n"
"        selectable_items[i] = history[#history + 1 - i]\n"
"    end\n"
"\n"
"    calculate_max_item_width()\n"
"    handle_edit()\n"
"    bind_mouse()\n"
"end\n"
"\n"
"local function page_up_or_prev_char()\n"
"    if selectable_items then\n"
"        handle_pgup()\n"
"    else\n"
"        prev_char()\n"
"    end\n"
"end\n"
"\n"
"local function page_down_or_next_char()\n"
"    if selectable_items then\n"
"        handle_pgdown()\n"
"    else\n"
"        next_char()\n"
"    end\n"
"end\n"
"\n"
"-- Move to the start of the current word, or if already at the start, the start\n"
"-- of the previous word. (Ctrl+Left)\n"
"local function prev_word()\n"
"    -- This is basically the same as next_word() but backwards, so reverse the\n"
"    -- string in order to do a \"backwards\" find. This wouldn't be as annoying\n"
"    -- to do if Lua didn't insist on 1-based indexing.\n"
"    cursor = line:len() - select(2, line:reverse():find('%s*[^%s]*', line:len() - cursor + 2)) + 1\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Move to the end of the current word, or if already at the end, the end of\n"
"-- the next word. (Ctrl+Right)\n"
"local function next_word()\n"
"    cursor = select(2, line:find('%s*[^%s]*', cursor)) + 1\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Move the cursor to the beginning of the line (HOME)\n"
"local function go_home()\n"
"    cursor = 1\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Move the cursor to the end of the line (END)\n"
"local function go_end()\n"
"    cursor = line:len() + 1\n"
"    handle_cursor_move()\n"
"end\n"
"\n"
"-- Delete from the cursor to the beginning of the word (Ctrl+Backspace)\n"
"local function del_word()\n"
"    local before_cur = line:sub(1, cursor - 1)\n"
"    local after_cur = line:sub(cursor)\n"
"\n"
"    before_cur = before_cur:gsub('[^%s]+%s*\044', '', 1)\n"
"    line = before_cur .. after_cur\n"
"    cursor = before_cur:len() + 1\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Delete from the cursor to the end of the word (Ctrl+Del)\n"
"local function del_next_word()\n"
"    if cursor > line:len() then return end\n"
"\n"
"    local before_cur = line:sub(1, cursor - 1)\n"
"    local after_cur = line:sub(cursor)\n"
"\n"
"    after_cur = after_cur:gsub('^%s*[^%s]+', '', 1)\n"
"    line = before_cur .. after_cur\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Delete from the cursor to the end of the line (Ctrl+K)\n"
"local function del_to_eol()\n"
"    line = line:sub(1, cursor - 1)\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Delete from the cursor back to the start of the line (Ctrl+U)\n"
"local function del_to_start()\n"
"    line = line:sub(cursor)\n"
"    cursor = 1\n"
"    handle_edit()\n"
"end\n"
"\n"
"-- Empty the log buffer of all messages (Ctrl+L)\n"
"local function clear_log_buffer()\n"
"    log_buffers[id] = {}\n"
"    render()\n"
"end\n"
"\n"
"-- Returns a string of UTF-8 text from the clipboard (or the primary selection)\n"
"local function get_clipboard(clip)\n"
"    if platform == 'x11' then\n"
"        local res = utils.subprocess({\n"
"            args = { 'xclip', '-selection', clip and 'clipboard' or 'primary', '-out' },\n"
"            playback_only = false,\n"
"        })\n"
"        if not res.error then\n"
"            return res.stdout\n"
"        end\n"
"    elseif platform == 'wayland' then\n"
"        if mp.get_property('current-clipboard-backend') == 'wayland' then\n"
"            local property = clip and 'clipboard/text' or 'clipboard/text-primary'\n"
"            return mp.get_property(property, '')\n"
"        end\n"
"        -- Wayland VO clipboard is only updated on window focus\n"
"        if clip and get_property_cached('focused') then\n"
"            return mp.get_property('clipboard/text', '')\n"
"        end\n"
"        local res = utils.subprocess({\n"
"            args = { 'wl-paste', clip and '-n' or  '-np' },\n"
"            playback_only = false,\n"
"        })\n"
"        if not res.error then\n"
"            return res.stdout\n"
"        end\n"
"    elseif platform == 'windows' or platform == 'darwin' then\n"
"        return mp.get_property('clipboard/text', '')\n"
"    end\n"
"    return ''\n"
"end\n"
"\n"
"-- Paste text from the window-system's clipboard. 'clip' determines whether the\n"
"-- clipboard or the primary selection buffer is used (on X11 and Wayland only.)\n"
"local function paste(clip)\n"
"    local text = get_clipboard(clip)\n"
"    local before_cur = line:sub(1, cursor - 1)\n"
"    local after_cur = line:sub(cursor)\n"
"    line = before_cur .. text .. after_cur\n"
"    cursor = cursor + text:len()\n"
"    handle_edit()\n"
"end\n"
"\n"
"local function text_input(info)\n"
"    if info.key_text and (info.event == \"press\" or info.event == \"down\"\n"
"                          or info.event == \"repeat\")\n"
"    then\n"
"        handle_char_input(info.key_text)\n"
"    end\n"
"end\n"
"\n"
"\n"
"local function common_prefix_length(s1, s2)\n"
"    local common_count = 0\n"
"    for i = 1, #s1 do\n"
"        if s1:byte(i) ~= s2:byte(i) then\n"
"            break\n"
"        end\n"
"        common_count = common_count + 1\n"
"    end\n"
"    return common_count\n"
"end\n"
"\n"
"local function max_overlap_length(s1, s2)\n"
"    for s1_offset = 0, #s1 - 1 do\n"
"        local match = true\n"
"        for i = 1, #s1 - s1_offset do\n"
"            if s1:byte(s1_offset + i) ~= s2:byte(i) then\n"
"                match = false\n"
"                break\n"
"            end\n"
"        end\n"
"        if match then\n"
"            return #s1 - s1_offset\n"
"        end\n"
"    end\n"
"    return 0\n"
"end\n"
"\n"
"-- If str starts with the first or last characters of prefix, strip them.\n"
"local function strip_common_characters(str, prefix)\n"
"    return str:sub(1 + math.max(\n"
"    common_prefix_length(prefix, str),\n"
"    max_overlap_length(prefix, str)))\n"
"end\n"
"\n"
"cycle_through_completions = function (backwards)\n"
"    if #completion_buffer == 0 then\n"
"        -- Allow Tab completion of commands before typing anything.\n"
"        if line == '' then\n"
"            complete()\n"
"        end\n"
"\n"
"        return\n"
"    end\n"
"\n"
"    selected_completion_index = selected_completion_index + (backwards and -1 or 1)\n"
"\n"
"    if selected_completion_index > #completion_buffer then\n"
"        selected_completion_index = 1\n"
"    elseif selected_completion_index < 1 then\n"
"        selected_completion_index = #completion_buffer\n"
"    end\n"
"\n"
"    local before_cur = line:sub(1, completion_pos - 1) ..\n"
"                       completion_buffer[selected_completion_index] .. completion_append\n"
"    line = before_cur .. strip_common_characters(line:sub(cursor),\n"
"        completion_buffer[selected_completion_index] .. completion_append)\n"
"    cursor = before_cur:len() + 1\n"
"    render()\n"
"end\n"
"\n"
"-- Show autocompletions.\n"
"complete = function ()\n"
"    completion_old_line = line\n"
"    completion_old_cursor = cursor\n"
"    mp.commandv('script-message-to', input_caller, 'input-event',\n"
"                'complete', utils.format_json({line:sub(1, cursor - 1)}))\n"
"    render()\n"
"end\n"
"\n"
"-- List of input bindings. This is a weird mashup between common GUI text-input\n"
"-- bindings and readline bindings.\n"
"local function get_bindings()\n"
"    local bindings = {\n"
"        { 'esc',         function() set_active(false) end       },\n"
"        { 'ctrl+[',      function() set_active(false) end       },\n"
"        { 'enter',       submit                                 },\n"
"        { 'kp_enter',    submit                                 },\n"
"        { 'shift+enter', function() handle_char_input('\\n') end },\n"
"        { 'ctrl+j',      submit                                 },\n"
"        { 'ctrl+m',      submit                                 },\n"
"        { 'bs',          handle_backspace                       },\n"
"        { 'shift+bs',    handle_backspace                       },\n"
"        { 'ctrl+h',      handle_backspace                       },\n"
"        { 'del',         handle_del                             },\n"
"        { 'shift+del',   handle_del                             },\n"
"        { 'ins',         handle_ins                             },\n"
"        { 'shift+ins',   function() paste(false) end            },\n"
"        { 'mbtn_mid',    function() paste(false) end            },\n"
"        { 'left',        function() prev_char() end             },\n"
"        { 'ctrl+b',      function() page_up_or_prev_char() end  },\n"
"        { 'right',       function() next_char() end             },\n"
"        { 'ctrl+f',      function() page_down_or_next_char() end},\n"
"        { 'up',          function() move_history(-1) end        },\n"
"        { 'ctrl+p',      function() move_history(-1) end        },\n"
"        { 'wheel_up',    function() move_history(-1, true) end  },\n"
"        { 'down',        function() move_history(1) end         },\n"
"        { 'ctrl+n',      function() move_history(1) end         },\n"
"        { 'wheel_down',  function() move_history(1, true) end   },\n"
"        { 'wheel_left',  function() end                         },\n"
"        { 'wheel_right', function() end                         },\n"
"        { 'ctrl+left',   prev_word                              },\n"
"        { 'alt+b',       prev_word                              },\n"
"        { 'ctrl+right',  next_word                              },\n"
"        { 'alt+f',       next_word                              },\n"
"        { 'tab',         cycle_through_completions              },\n"
"        { 'ctrl+i',      cycle_through_completions              },\n"
"        { 'shift+tab',   function() cycle_through_completions(true) end },\n"
"        { 'ctrl+a',      go_home                                },\n"
"        { 'home',        go_home                                },\n"
"        { 'ctrl+e',      go_end                                 },\n"
"        { 'end',         go_end                                 },\n"
"        { 'pgup',        handle_pgup                            },\n"
"        { 'pgdwn',       handle_pgdown                          },\n"
"        { 'ctrl+r',      search_history                         },\n"
"        { 'ctrl+c',      clear                                  },\n"
"        { 'ctrl+d',      maybe_exit                             },\n"
"        { 'ctrl+k',      del_to_eol                             },\n"
"        { 'ctrl+l',      clear_log_buffer                       },\n"
"        { 'ctrl+u',      del_to_start                           },\n"
"        { 'ctrl+v',      function() paste(true) end             },\n"
"        { 'meta+v',      function() paste(true) end             },\n"
"        { 'ctrl+bs',     del_word                               },\n"
"        { 'ctrl+w',      del_word                               },\n"
"        { 'ctrl+del',    del_next_word                          },\n"
"        { 'alt+d',       del_next_word                          },\n"
"        { 'kp_dec',      function() handle_char_input('.') end  },\n"
"        { 'kp_add',      function() handle_char_input('+') end  },\n"
"        { 'kp_subtract', function() handle_char_input('-') end  },\n"
"        { 'kp_multiply', function() handle_char_input('*') end  },\n"
"        { 'kp_divide',   function() handle_char_input('/') end  },\n"
"    }\n"
"\n"
"    for i = 0, 9 do\n"
"        bindings[#bindings + 1] =\n"
"            {'kp' .. i, function() handle_char_input('' .. i) end}\n"
"    end\n"
"\n"
"    return bindings\n"
"end\n"
"\n"
"local function define_key_bindings()\n"
"    if #key_bindings > 0 then\n"
"        return\n"
"    end\n"
"    for _, bind in ipairs(get_bindings()) do\n"
"        if not (dont_bind_up_down and (bind[1] == 'up' or bind[1] == 'down')) then\n"
"            -- Generate arbitrary name for removing the bindings later.\n"
"            local name = \"_console_\" .. (#key_bindings + 1)\n"
"            key_bindings[#key_bindings + 1] = name\n"
"            mp.add_forced_key_binding(bind[1], name, bind[2], {repeatable = true})\n"
"        end\n"
"    end\n"
"    mp.add_forced_key_binding(\"any_unicode\", \"_console_text\", text_input,\n"
"        {repeatable = true, complex = true})\n"
"    key_bindings[#key_bindings + 1] = \"_console_text\"\n"
"end\n"
"\n"
"local function undefine_key_bindings()\n"
"    for _, name in ipairs(key_bindings) do\n"
"        mp.remove_key_binding(name)\n"
"    end\n"
"    key_bindings = {}\n"
"end\n"
"\n"
"local function read_history()\n"
"    if not history_paths[id] or history[1] then\n"
"        return\n"
"    end\n"
"\n"
"    local history_file = io.open(mp.command_native({'expand-path', history_paths[id]}))\n"
"\n"
"    if history_file == nil then\n"
"        return\n"
"    end\n"
"\n"
"    if opts.history_dedup then\n"
"        local unfiltered_history = {}\n"
"        for command in history_file:lines() do\n"
"            unfiltered_history[#unfiltered_history + 1] = command\n"
"        end\n"
"\n"
"        local history_map = {}\n"
"        for i = #unfiltered_history, 1, -1 do\n"
"            local command = unfiltered_history[i]\n"
"            if not history_map[command] then\n"
"                history[#history + 1] = command\n"
"                history_map[command] = true\n"
"            end\n"
"        end\n"
"\n"
"        for i = 1, #history / 2, 1 do\n"
"            history[i], history[#history - i + 1] = history[#history - i + 1], history[i]\n"
"        end\n"
"    else\n"
"        for command in history_file:lines() do\n"
"            history[#history + 1] = command\n"
"        end\n"
"    end\n"
"\n"
"    history_file:close()\n"
"end\n"
"\n"
"-- Open or close the console\n"
"set_active = function (active)\n"
"    if active == open then\n"
"        return\n"
"    end\n"
"\n"
"    if active then\n"
"        open = true\n"
"        insert_mode = false\n"
"        define_key_bindings()\n"
"        mp.set_property_bool('user-data/mpv/console/open', true)\n"
"        ime_active = mp.get_property_bool('input-ime')\n"
"        mp.set_property_bool('input-ime', true)\n"
"    elseif searching_history then\n"
"        searching_history = false\n"
"        selectable_items = nil\n"
"        unbind_mouse()\n"
"    else\n"
"        open = false\n"
"        undefine_key_bindings()\n"
"        unbind_mouse()\n"
"        mp.set_property_bool('user-data/mpv/console/open', false)\n"
"        mp.set_property_bool('input-ime', ime_active)\n"
"        mp.commandv('script-message-to', input_caller, 'input-event',\n"
"                    'closed', utils.format_json({line, cursor}))\n"
"        collectgarbage()\n"
"    end\n"
"    render()\n"
"end\n"
"\n"
"mp.register_script_message('disable', function()\n"
"    set_active(false)\n"
"end)\n"
"\n"
"mp.register_script_message('get-input', function (script_name, args)\n"
"    if open and script_name ~= input_caller then\n"
"        mp.commandv('script-message-to', input_caller, 'input-event',\n"
"                    'closed', utils.format_json({line, cursor}))\n"
"    end\n"
"\n"
"    input_caller = script_name\n"
"    args = utils.parse_json(args)\n"
"    prompt = args.prompt\n"
"    line = args.default_text or ''\n"
"    cursor = tonumber(args.cursor_position) or line:len() + 1\n"
"    keep_open = args.keep_open\n"
"    default_item = args.default_item\n"
"    has_completions = args.has_completions\n"
"    dont_bind_up_down = args.dont_bind_up_down\n"
"    searching_history = false\n"
"\n"
"    if args.items then\n"
"        selectable_items = {}\n"
"\n"
"        -- Limit the number of characters to prevent libass from freezing mpv.\n"
"        -- Not important for terminal output.\n"
"        local limit = terminal_output() and 5000 or (5 * osd_width / opts.font_size)\n"
"\n"
"        for i, item in ipairs(args.items) do\n"
"            selectable_items[i] = item:gsub(\"[\\r\\n].*\", \"\342\213\257\"):sub(1, limit)\n"
"        end\n"
"\n"
"        calculate_max_item_width()\n"
"        handle_edit()\n"
"        bind_mouse()\n"
"    else\n"
"        selectable_items = nil\n"
"        unbind_mouse()\n"
"        id = args.id or script_name .. prompt\n"
"        completion_buffer = {}\n"
"        autoselect_completion = args.autoselect_completion\n"
"\n"
"        if histories[id] == nil then\n"
"            histories[id] = {}\n"
"            log_buffers[id] = {}\n"
"            histories_to_save[id] = ''\n"
"        end\n"
"        history = histories[id]\n"
"        history_paths[id] = args.history_path\n"
"        read_history()\n"
"        history_pos = #history + 1\n"
"\n"
"        if line ~= '' then\n"
"            complete()\n"
"        end\n"
"    end\n"
"\n"
"    set_active(true)\n"
"    mp.commandv('script-message-to', input_caller, 'input-event', 'opened')\n"
"end)\n"
"\n"
"-- Add a line to the log buffer (which is limited to 100 lines)\n"
"mp.register_script_message('log', function (message)\n"
"    local log_buffer = log_buffers[id]\n"
"    message = utils.parse_json(message)\n"
"\n"
"    log_buffer[#log_buffer + 1] = {\n"
"        text = message.text,\n"
"        style = message.error and styles.error or message.style or '',\n"
"        terminal_style = message.error and terminal_styles.error or\n"
"                         message.terminal_style or '',\n"
"    }\n"
"\n"
"    if #log_buffer > 100 then\n"
"        table.remove(log_buffer, 1)\n"
"    end\n"
"\n"
"    if not open then\n"
"        return\n"
"    end\n"
"\n"
"    if not update_timer:is_enabled() then\n"
"        render()\n"
"        update_timer:resume()\n"
"    else\n"
"        pending_update = true\n"
"    end\n"
"end)\n"
"\n"
"mp.register_script_message('set-log', function (log)\n"
"    log = utils.parse_json(log)\n"
"    log_buffers[id] = {}\n"
"\n"
"    for i = 1, #log do\n"
"        if type(log[i]) == 'table' then\n"
"            log[i].text = log[i].text\n"
"            log[i].style = log[i].style or ''\n"
"            log[i].terminal_style = log[i].terminal_style or ''\n"
"            log_buffers[id][i] = log[i]\n"
"        else\n"
"            log_buffers[id][i] = {\n"
"                text = log[i],\n"
"                style = '',\n"
"                terminal_style = '',\n"
"            }\n"
"        end\n"
"    end\n"
"\n"
"    render()\n"
"end)\n"
"\n"
"mp.register_script_message('complete', function (list, start_pos, append)\n"
"    if line ~= completion_old_line or cursor ~= completion_old_cursor then\n"
"        return\n"
"    end\n"
"\n"
"    completion_buffer = {}\n"
"    selected_completion_index = 0\n"
"    local completions = utils.parse_json(list)\n"
"    table.sort(completions)\n"
"    completion_pos = start_pos\n"
"    completion_append = append\n"
"    for i, match in ipairs(fuzzy_find(line:sub(completion_pos, cursor - 1),\n"
"                                      completions)) do\n"
"        completion_buffer[i] = completions[match[1]]\n"
"    end\n"
"\n"
"    render()\n"
"end)\n"
"\n"
"local function resize()\n"
"    calculate_max_item_width()\n"
"    render()\n"
"end\n"
"\n"
"mp.observe_property('osd-dimensions', 'native', function (_, value)\n"
"    if value.w == osd_width and value.h == osd_height then\n"
"        return\n"
"    end\n"
"    osd_width, osd_height = value.w, value.h\n"
"    resize()\n"
"end)\n"
"\n"
"mp.observe_property('display-hidpi-scale', 'native', function (name, value)\n"
"    property_cache[name] = value\n"
"    resize()\n"
"end)\n"
"\n"
"mp.observe_property('focused', 'native', function (name, value)\n"
"    property_cache[name] = value\n"
"    render()\n"
"end)\n"
"\n"
"mp.observe_property(\"user-data/osc/margins\", \"native\", function(_, val)\n"
"    if type(val) == \"table\" and type(val.t) == \"number\" and type(val.b) == \"number\" then\n"
"        global_margins = val\n"
"    else\n"
"        global_margins = { t = 0, b = 0 }\n"
"    end\n"
"    render()\n"
"end)\n"
"\n"
"mp.register_event('shutdown', function ()\n"
"    mp.del_property('user-data/mpv/console')\n"
"\n"
"    for history_id, history_path in pairs(history_paths) do\n"
"        history_path = mp.command_native({'expand-path', history_path})\n"
"        local history_file, error_message = io.open(history_path, 'ab')\n"
"\n"
"        if history_file then\n"
"            history_file:write(histories_to_save[history_id])\n"
"            history_file:close()\n"
"        else\n"
"            mp.msg.error('Failed to write history: ' .. error_message)\n"
"        end\n"
"    end\n"
"end)\n"
"\n"
"-- These are for backwards compatibility only.\n"
"mp.add_key_binding(nil, 'enable', function ()\n"
"    mp.command('script-message-to commands open')\n"
"end)\n"
"\n"
"mp.register_script_message('type', function (text, cursor_pos)\n"
"    mp.commandv('script-message-to', 'commands', 'type', unpack({text, cursor_pos}))\n"
"end)\n"
"\n"
"require 'mp.options'.read_options(opts, nil, render)\n"
"\n"
"collectgarbage()\n"
