// Generated from player/lua/positioning.lua

"--[[\n"
"This file is part of mpv.\n"
"\n"
"mpv is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU Lesser General Public\n"
"License as published by the Free Software Foundation; either\n"
"version 2.1 of the License, or (at your option) any later version.\n"
"\n"
"mpv is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU Lesser General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU Lesser General Public\n"
"License along with mpv.  If not, see <http://www.gnu.org/licenses/>.\n"
"]]\n"
"\n"
"local options = {\n"
"    toggle_align_to_cursor = false,\n"
"    suppress_osd = false,\n"
"}\n"
"\n"
"require \"mp.options\".read_options(options, nil, function () end)\n"
"\n"
"local function clamp(value, min, max)\n"
"    return math.min(math.max(value, min), max)\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"pan-x\", function (t)\n"
"    if t.event == \"up\" then\n"
"        return\n"
"    end\n"
"\n"
"    local dims = mp.get_property_native(\"osd-dimensions\")\n"
"    local width = dims.w - dims.ml - dims.mr\n"
"    -- 1 video-align shifts the OSD by (width - osd-width) / 2 pixels, so the\n"
"    -- equation to find how much video-align to add to offset the OSD by\n"
"    -- osd-width is:\n"
"    -- x/1 = osd-width / ((width - osd-width) / 2)\n"
"    local amount = t.arg * t.scale * 2 * dims.w / (width - dims.w)\n"
"    mp.commandv(\"add\", \"video-align-x\", clamp(amount, -2, 2))\n"
"end, { complex = true, scalable = true })\n"
"\n"
"mp.add_key_binding(nil, \"pan-y\", function (t)\n"
"    if t.event == \"up\" then\n"
"        return\n"
"    end\n"
"\n"
"    local dims = mp.get_property_native(\"osd-dimensions\")\n"
"    local height = dims.h - dims.mt - dims.mb\n"
"    local amount = t.arg * t.scale * 2 * dims.h / (height - dims.h)\n"
"    mp.commandv(\"add\", \"video-align-y\", clamp(amount, -2, 2))\n"
"end, { complex = true, scalable = true })\n"
"\n"
"mp.add_key_binding(nil, \"drag-to-pan\", function (t)\n"
"    if t.event == \"up\" then\n"
"        mp.remove_key_binding(\"drag-to-pan-mouse-move\")\n"
"        return\n"
"    end\n"
"\n"
"    local dims = mp.get_property_native(\"osd-dimensions\")\n"
"    local old_mouse_pos = mp.get_property_native(\"mouse-pos\")\n"
"    local old_align_x = mp.get_property_native(\"video-align-x\")\n"
"    local old_align_y = mp.get_property_native(\"video-align-y\")\n"
"\n"
"    mp.add_forced_key_binding(\"MOUSE_MOVE\", \"drag-to-pan-mouse-move\", function ()\n"
"        local mouse_pos = mp.get_property_native(\"mouse-pos\")\n"
"        -- 1 video-align shifts the OSD by (dimension - osd_dimension) / 2 pixels,\n"
"        -- so the equation to find how much video-align to add to offset the OSD\n"
"        -- by the difference in mouse position is:\n"
"        -- x/1 = (mouse_pos - old_mouse_pos) / ((dimension - osd_dimension) / 2)\n"
"        local align = old_align_x + 2 * (mouse_pos.x - old_mouse_pos.x)\n"
"                      / (dims.ml + dims.mr)\n"
"        mp.set_property(\"video-align-x\", clamp(align, -1, 1))\n"
"        align = old_align_y + 2 * (mouse_pos.y - old_mouse_pos.y)\n"
"                / (dims.mt + dims.mb)\n"
"        mp.set_property(\"video-align-y\", clamp(align, -1, 1))\n"
"    end)\n"
"end, { complex = true })\n"
"\n"
"\n"
"local align_to_cursor_bound = false\n"
"\n"
"local function align_to_cursor(_, mouse_pos)\n"
"    local dims = mp.get_property_native(\"osd-dimensions\")\n"
"    local align = (mouse_pos.x * 2 - dims.w) / dims.w\n"
"    mp.set_property(\"video-align-x\", clamp(align, -1, 1))\n"
"    align = (mouse_pos.y * 2 - dims.h) / dims.h\n"
"    mp.set_property(\"video-align-y\", clamp(align, -1, 1))\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"align-to-cursor\", function (t)\n"
"    if options.toggle_align_to_cursor == false then\n"
"        if t.event == \"down\" then\n"
"            mp.observe_property(\"mouse-pos\", \"native\", align_to_cursor)\n"
"        else\n"
"            mp.unobserve_property(align_to_cursor)\n"
"        end\n"
"\n"
"        return\n"
"    end\n"
"\n"
"    if t.event ~= \"up\" then\n"
"        return\n"
"    end\n"
"\n"
"    if align_to_cursor_bound then\n"
"        mp.unobserve_property(align_to_cursor)\n"
"    else\n"
"        mp.observe_property(\"mouse-pos\", \"native\", align_to_cursor)\n"
"    end\n"
"    align_to_cursor_bound = not align_to_cursor_bound\n"
"end, { complex = true })\n"
"\n"
"\n"
"mp.add_key_binding(nil, \"cursor-centric-zoom\", function (t)\n"
"    local amount = t.arg * t.scale\n"
"\n"
"    local command = (options.suppress_osd and \"no-osd \" or \"\") ..\n"
"                    \"add video-zoom \" .. amount .. \";\"\n"
"\n"
"    local x, y\n"
"    local touch_positions = mp.get_property_native(\"touch-pos\")\n"
"    if touch_positions[1] then\n"
"        for _, position in pairs(touch_positions) do\n"
"            x = x + position.x\n"
"            y = y + position.y\n"
"        end\n"
"        x = x / #touch_positions\n"
"        y = y / #touch_positions\n"
"    else\n"
"        local mouse_pos = mp.get_property_native(\"mouse-pos\")\n"
"        x = mouse_pos.x\n"
"        y = mouse_pos.y\n"
"    end\n"
"\n"
"    local dims = mp.get_property_native(\"osd-dimensions\")\n"
"    local width = (dims.w - dims.ml - dims.mr) * 2^amount\n"
"    local height = (dims.h - dims.mt - dims.mb) * 2^amount\n"
"\n"
"    local old_cursor_ml = dims.ml - x\n"
"    local cursor_ml = old_cursor_ml * 2^amount\n"
"    local ml = cursor_ml + x\n"
"    -- video/out/aspect.c:src_dst_split_scaling() defines ml as:\n"
"    -- ml = (osd-width - width) * (video-align-x + 1) / 2 + pan-x * width\n"
"    -- So video-align-x is:\n"
"    local align = 2 * (ml - mp.get_property_native(\"video-pan-x\") * width)\n"
"                  / (dims.w - width) - 1\n"
"    command = command .. \"no-osd set video-align-x \" .. clamp(align, -1, 1) .. \";\"\n"
"\n"
"    local mt = (dims.mt - y) * 2^amount + y\n"
"    align = 2 * (mt - mp.get_property_native(\"video-pan-y\") * height)\n"
"            / (dims.h - height) - 1\n"
"    command = command .. \"no-osd set video-align-y \" .. clamp(align, -1, 1)\n"
"\n"
"    mp.command(command)\n"
"end, { complex = true, scalable = true })\n"
