// Generated from player/lua/select.lua

"--[[\n"
"This file is part of mpv.\n"
"\n"
"mpv is free software; you can redistribute it and/or\n"
"modify it under the terms of the GNU Lesser General Public\n"
"License as published by the Free Software Foundation; either\n"
"version 2.1 of the License, or (at your option) any later version.\n"
"\n"
"mpv is distributed in the hope that it will be useful,\n"
"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
"GNU Lesser General Public License for more details.\n"
"\n"
"You should have received a copy of the GNU Lesser General Public\n"
"License along with mpv.  If not, see <http://www.gnu.org/licenses/>.\n"
"]]\n"
"\n"
"local utils = require \"mp.utils\"\n"
"local input = require \"mp.input\"\n"
"\n"
"local options = {\n"
"    history_date_format = \"%Y-%m-%d %H:%M:%S\",\n"
"    hide_history_duplicates = true,\n"
"}\n"
"\n"
"require \"mp.options\".read_options(options, nil, function () end)\n"
"\n"
"local function show_warning(message)\n"
"    mp.msg.warn(message)\n"
"    if mp.get_property_native(\"vo-configured\") then\n"
"        mp.osd_message(message)\n"
"    end\n"
"end\n"
"\n"
"local function show_error(message)\n"
"    mp.msg.error(message)\n"
"    if mp.get_property_native(\"vo-configured\") then\n"
"        mp.osd_message(message)\n"
"    end\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-playlist\", function ()\n"
"    local playlist = {}\n"
"    local default_item\n"
"    local show = mp.get_property_native(\"osd-playlist-entry\")\n"
"    local trailing_slash_pattern = mp.get_property(\"platform\") == \"windows\"\n"
"                                   and \"[/\\\\]+\044\" or \"/+\044\"\n"
"\n"
"    for i, entry in ipairs(mp.get_property_native(\"playlist\")) do\n"
"        playlist[i] = entry.title\n"
"        if not playlist[i] or show ~= \"title\" then\n"
"            playlist[i] = entry.filename\n"
"            if not playlist[i]:find(\"://\") then\n"
"                playlist[i] = select(2, utils.split_path(\n"
"                    playlist[i]:gsub(trailing_slash_pattern, \"\")))\n"
"            end\n"
"        end\n"
"        if entry.title and show == \"both\" then\n"
"            playlist[i] = string.format(\"%s (%s)\", entry.title, playlist[i])\n"
"        end\n"
"\n"
"        if entry.playing then\n"
"            default_item = i\n"
"        end\n"
"    end\n"
"\n"
"    if #playlist == 0 then\n"
"        show_warning(\"The playlist is empty.\")\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a playlist entry:\",\n"
"        items = playlist,\n"
"        default_item = default_item,\n"
"        submit = function (index)\n"
"            mp.commandv(\"playlist-play-index\", index - 1)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local function format_flags(track)\n"
"    local flags = \"\"\n"
"\n"
"    for _, flag in ipairs({\n"
"        \"default\", \"forced\", \"dependent\", \"visual-impaired\", \"hearing-impaired\",\n"
"        \"image\", \"external\"\n"
"    }) do\n"
"        if track[flag] then\n"
"            flags = flags .. flag .. \" \"\n"
"        end\n"
"    end\n"
"\n"
"    if flags == \"\" then\n"
"        return \"\"\n"
"    end\n"
"\n"
"    return \" [\" .. flags:sub(1, -2) .. \"]\"\n"
"end\n"
"\n"
"local function format_track(track)\n"
"    local bitrate = track[\"demux-bitrate\"] or track[\"hls-bitrate\"]\n"
"\n"
"    return (track.selected and \"\342\227\217\" or \"\342\227\213\") ..\n"
"        (track.title and \" \" .. track.title or \"\") ..\n"
"        \" (\" .. (\n"
"            (track.lang and track.lang .. \" \" or \"\") ..\n"
"            (track.codec and track.codec .. \" \" or \"\") ..\n"
"            (track[\"demux-w\"] and track[\"demux-w\"] .. \"x\" .. track[\"demux-h\"]\n"
"             .. \" \" or \"\") ..\n"
"            (track[\"demux-fps\"] and not track.image\n"
"             and string.format(\"%.4f\", track[\"demux-fps\"]):gsub(\"%.\0770*\044\", \"\") ..\n"
"             \" fps \" or \"\") ..\n"
"            (track[\"demux-channel-count\"] and track[\"demux-channel-count\"] ..\n"
"             \"ch \" or \"\") ..\n"
"            (track[\"codec-profile\"] and track.type == \"audio\"\n"
"             and track[\"codec-profile\"] .. \" \" or \"\") ..\n"
"            (track[\"demux-samplerate\"] and track[\"demux-samplerate\"] / 1000 ..\n"
"             \" kHz \" or \"\") ..\n"
"            (bitrate and string.format(\"%.0f\", bitrate / 1000) ..\n"
"             \" kbps \" or \"\")\n"
"        ):sub(1, -2) .. \")\" .. format_flags(track)\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-track\", function ()\n"
"    local tracks = {}\n"
"\n"
"    for i, track in ipairs(mp.get_property_native(\"track-list\")) do\n"
"        tracks[i] = (track.image and \"Image\" or\n"
"                     track.type:sub(1, 1):upper() .. track.type:sub(2)) .. \": \" ..\n"
"                    format_track(track)\n"
"    end\n"
"\n"
"    if #tracks == 0 then\n"
"        show_warning(\"No available tracks.\")\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a track:\",\n"
"        items = tracks,\n"
"        submit = function (id)\n"
"            local track = mp.get_property_native(\"track-list/\" .. id - 1)\n"
"            if track then\n"
"                mp.set_property(track.type, track.selected and \"no\" or track.id)\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local function select_track(property, type, prompt, warning)\n"
"    local tracks = {}\n"
"    local items = {}\n"
"    local default_item\n"
"    local track_id = mp.get_property_native(property)\n"
"\n"
"    for _, track in ipairs(mp.get_property_native(\"track-list\")) do\n"
"        if track.type == type then\n"
"            tracks[#tracks + 1] = track\n"
"            items[#items + 1] = format_track(track)\n"
"\n"
"            if track.id == track_id then\n"
"                default_item = #items\n"
"            end\n"
"        end\n"
"    end\n"
"\n"
"    if #items == 0 then\n"
"        show_warning(warning)\n"
"        return\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = prompt,\n"
"        items = items,\n"
"        default_item = default_item,\n"
"        submit = function (id)\n"
"            mp.set_property(property, tracks[id].selected and \"no\" or tracks[id].id)\n"
"        end,\n"
"    })\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-sid\", function ()\n"
"    select_track(\"sid\", \"sub\", \"Select a subtitle:\", \"No available subtitles.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-secondary-sid\", function ()\n"
"    select_track(\"secondary-sid\", \"sub\", \"Select a secondary subtitle:\",\n"
"                 \"No available subtitles.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-aid\", function ()\n"
"    select_track(\"aid\", \"audio\", \"Select an audio track:\",\n"
"                 \"No available audio tracks.\")\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-vid\", function ()\n"
"    select_track(\"vid\", \"video\", \"Select a video track:\",\n"
"                 \"No available video tracks.\")\n"
"end)\n"
"\n"
"local function format_time(t, duration)\n"
"    local h = math.floor(t / (60 * 60))\n"
"    t = t - (h * 60 * 60)\n"
"    local m = math.floor(t / 60)\n"
"    local s = t - (m * 60)\n"
"\n"
"    if duration >= 60 * 60 or h > 0 then\n"
"        return string.format(\"%.2d:%.2d:%.2d\", h, m, s)\n"
"    end\n"
"\n"
"    return string.format(\"%.2d:%.2d\", m, s)\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-chapter\", function ()\n"
"    local chapters = {}\n"
"    local default_item = mp.get_property_native(\"chapter\")\n"
"\n"
"    if default_item == nil then\n"
"        show_warning(\"No available chapters.\")\n"
"        return\n"
"    end\n"
"\n"
"    local duration = mp.get_property_native(\"duration\", math.huge)\n"
"\n"
"    for i, chapter in ipairs(mp.get_property_native(\"chapter-list\")) do\n"
"        chapters[i] = format_time(chapter.time, duration) .. \" \" .. chapter.title\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a chapter:\",\n"
"        items = chapters,\n"
"        default_item = default_item > -1 and default_item + 1,\n"
"        submit = function (chapter)\n"
"            mp.set_property(\"chapter\", chapter - 1)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-edition\", function ()\n"
"    local edition_list = mp.get_property_native(\"edition-list\")\n"
"\n"
"    if edition_list == nil or #edition_list < 2 then\n"
"        show_warning(\"No available editions.\")\n"
"        return\n"
"    end\n"
"\n"
"    local editions = {}\n"
"\n"
"    for i, edition in ipairs(edition_list) do\n"
"        editions[i] = edition.title or (\"Edition \" .. edition.id + 1)\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select an edition:\",\n"
"        items = editions,\n"
"        default_item = mp.get_property_native(\"current-edition\") + 1,\n"
"        submit = function (edition)\n"
"            mp.set_property(\"edition\", edition - 1)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-subtitle-line\", function ()\n"
"    local sub = mp.get_property_native(\"current-tracks/sub\")\n"
"\n"
"    if sub == nil then\n"
"        show_warning(\"No subtitle is loaded.\")\n"
"        return\n"
"    end\n"
"\n"
"    if sub.external and sub[\"external-filename\"]:find(\"^edl://\") then\n"
"        sub[\"external-filename\"] = sub[\"external-filename\"]:match('https\077://.*')\n"
"                                   or sub[\"external-filename\"]\n"
"    end\n"
"\n"
"    local r = mp.command_native({\n"
"        name = \"subprocess\",\n"
"        capture_stdout = true,\n"
"        args = sub.external\n"
"            and {\"ffmpeg\", \"-loglevel\", \"error\", \"-i\", sub[\"external-filename\"],\n"
"                 \"-f\", \"lrc\", \"-map_metadata\", \"-1\", \"-fflags\", \"+bitexact\", \"-\"}\n"
"            or {\"ffmpeg\", \"-loglevel\", \"error\", \"-i\", mp.get_property(\"path\"),\n"
"                \"-map\", \"s:\" .. sub[\"id\"] - 1, \"-f\", \"lrc\", \"-map_metadata\",\n"
"                \"-1\", \"-fflags\", \"+bitexact\", \"-\"}\n"
"    })\n"
"\n"
"    if r.error_string == \"init\" then\n"
"        show_error(\"Failed to extract subtitles: ffmpeg not found.\")\n"
"        return\n"
"    elseif r.status ~= 0 then\n"
"        show_error(\"Failed to extract subtitles.\")\n"
"        return\n"
"    end\n"
"\n"
"    local sub_lines = {}\n"
"    local sub_times = {}\n"
"    local default_item\n"
"    local delay = mp.get_property_native(\"sub-delay\")\n"
"    local time_pos = mp.get_property_native(\"time-pos\") - delay\n"
"    local duration = mp.get_property_native(\"duration\", math.huge)\n"
"\n"
"    -- Strip HTML and ASS tags.\n"
"    for line in r.stdout:gsub(\"<.->\", \"\"):gsub(\"{\\\\.-}\", \"\"):gmatch(\"[^\\n]+\") do\n"
"        -- ffmpeg outputs LRCs with minutes > 60 instead of adding hours.\n"
"        sub_times[#sub_times + 1] = line:match(\"%d+\") * 60 + line:match(\":([%d%.]*)\")\n"
"        sub_lines[#sub_lines + 1] = format_time(sub_times[#sub_times], duration) ..\n"
"                                    \" \" .. line:gsub(\".*]\", \"\", 1)\n"
"\n"
"        if sub_times[#sub_times] <= time_pos then\n"
"            default_item = #sub_times\n"
"        end\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a line to seek to:\",\n"
"        items = sub_lines,\n"
"        default_item = default_item,\n"
"        submit = function (index)\n"
"            -- Add an offset to seek to the correct line while paused without a\n"
"            -- video track.\n"
"            if mp.get_property_native(\"current-tracks/video/image\") ~= false then\n"
"                delay = delay + 0.1\n"
"            end\n"
"\n"
"            mp.commandv(\"seek\", sub_times[index] + delay, \"absolute\")\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-audio-device\", function ()\n"
"    local devices = mp.get_property_native(\"audio-device-list\")\n"
"    local items = {}\n"
"    -- This is only useful if an --audio-device has been explicitly set,\n"
"    -- otherwise its value is just auto and there is no current-audio-device\n"
"    -- property.\n"
"    local selected_device = mp.get_property(\"audio-device\")\n"
"    local default_item\n"
"\n"
"    if #devices == 0 then\n"
"        show_warning(\"No available audio devices.\")\n"
"        return\n"
"    end\n"
"\n"
"    for i, device in ipairs(devices) do\n"
"        items[i] = device.name .. \" (\" .. device.description .. \")\"\n"
"\n"
"        if device.name == selected_device then\n"
"            default_item = i\n"
"        end\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select an audio device:\",\n"
"        items = items,\n"
"        default_item = default_item,\n"
"        submit = function (id)\n"
"            mp.set_property(\"audio-device\", devices[id].name)\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local function format_history_entry(entry)\n"
"    local status\n"
"    status, entry.time = pcall(os.date, options.history_date_format, entry.time)\n"
"\n"
"    if not status then\n"
"        mp.msg.warn(entry.time)\n"
"    end\n"
"\n"
"    local item = \"(\" .. entry.time .. \") \"\n"
"\n"
"    if entry.title then\n"
"        return item .. entry.title .. \" (\" .. entry.path .. \")\"\n"
"    end\n"
"\n"
"    if entry.path:find(\"://\") then\n"
"        return item .. entry.path\n"
"    end\n"
"\n"
"    local directory, filename = utils.split_path(entry.path)\n"
"\n"
"    return item .. filename .. \" (\" .. directory .. \")\"\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"select-watch-history\", function ()\n"
"    local history_file_path = mp.command_native(\n"
"        {\"expand-path\", mp.get_property(\"watch-history-path\")})\n"
"    local history_file, error_message = io.open(history_file_path)\n"
"    if not history_file then\n"
"        show_warning(mp.get_property_native(\"save-watch-history\")\n"
"                     and error_message\n"
"                     or \"Enable --save-watch-history to jump to recently played files.\")\n"
"        return\n"
"    end\n"
"\n"
"    local all_entries = {}\n"
"    local line_num = 1\n"
"    for line in history_file:lines() do\n"
"        local entry = utils.parse_json(line)\n"
"        if entry and entry.path then\n"
"            all_entries[#all_entries + 1] = entry\n"
"        else\n"
"            mp.msg.warn(history_file_path .. \": Parse error at line \" .. line_num)\n"
"        end\n"
"        line_num = line_num + 1\n"
"    end\n"
"    history_file:close()\n"
"\n"
"    local entries = {}\n"
"    local items = {}\n"
"    local seen = {}\n"
"\n"
"    for i = #all_entries, 1, -1 do\n"
"        local entry = all_entries[i]\n"
"        if not seen[entry.path] or not options.hide_history_duplicates then\n"
"            seen[entry.path] = true\n"
"            entries[#entries + 1] = entry\n"
"            items[#items + 1] = format_history_entry(entry)\n"
"        end\n"
"    end\n"
"\n"
"    items[#items+1] = \"Clear history\"\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a file:\",\n"
"        items = items,\n"
"        submit = function (i)\n"
"            if entries[i] then\n"
"                mp.commandv(\"loadfile\", entries[i].path)\n"
"                return\n"
"            end\n"
"\n"
"            error_message = select(2, os.remove(history_file_path))\n"
"            if error_message then\n"
"                show_error(error_message)\n"
"            else\n"
"                mp.osd_message(\"History cleared.\")\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-watch-later\", function ()\n"
"    local watch_later_dir = mp.get_property(\"current-watch-later-dir\")\n"
"\n"
"    if not watch_later_dir then\n"
"        show_warning(\"No watch later files found.\")\n"
"        return\n"
"    end\n"
"\n"
"    local watch_later_files = {}\n"
"\n"
"    for i, file in ipairs(utils.readdir(watch_later_dir, \"files\") or {}) do\n"
"        watch_later_files[i] = watch_later_dir .. \"/\" .. file\n"
"    end\n"
"\n"
"    if #watch_later_files == 0 then\n"
"        show_warning(\"No watch later files found.\")\n"
"        return\n"
"    end\n"
"\n"
"    local files = {}\n"
"    for _, watch_later_file in pairs(watch_later_files) do\n"
"        local file_handle = io.open(watch_later_file)\n"
"        if file_handle then\n"
"            local line = file_handle:read()\n"
"            if line and line ~= \"# redirect entry\" and line:find(\"^#\") then\n"
"                files[#files + 1] = {line:sub(3), utils.file_info(watch_later_file).mtime}\n"
"            end\n"
"            file_handle:close()\n"
"        end\n"
"    end\n"
"\n"
"    if #files == 0 then\n"
"        show_warning(mp.get_property_native(\"write-filename-in-watch-later-config\")\n"
"            and \"No watch later files found.\"\n"
"            or \"Enable --write-filename-in-watch-later-config to select recent files.\")\n"
"        return\n"
"    end\n"
"\n"
"    table.sort(files, function (i, j)\n"
"        return i[2] > j[2]\n"
"    end)\n"
"\n"
"    local items = {}\n"
"    for i, file in ipairs(files) do\n"
"        items[i] = os.date(\"(%Y-%m-%d) \", file[2]) .. file[1]\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a file:\",\n"
"        items = items,\n"
"        submit = function (i)\n"
"            mp.commandv(\"loadfile\", files[i][1])\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"select-binding\", function ()\n"
"    local bindings = {}\n"
"\n"
"    for _, binding in pairs(mp.get_property_native(\"input-bindings\")) do\n"
"        if binding.priority >= 0 and (\n"
"               bindings[binding.key] == nil or\n"
"               (bindings[binding.key].is_weak and not binding.is_weak) or\n"
"               (binding.is_weak == bindings[binding.key].is_weak and\n"
"                binding.priority > bindings[binding.key].priority)\n"
"        ) then\n"
"            bindings[binding.key] = binding\n"
"        end\n"
"    end\n"
"\n"
"    local items = {}\n"
"    for _, binding in pairs(bindings) do\n"
"        if binding.cmd ~= \"ignore\" then\n"
"            items[#items + 1] = binding.key .. \" \" .. binding.cmd\n"
"        end\n"
"    end\n"
"\n"
"    table.sort(items)\n"
"\n"
"    input.select({\n"
"        prompt = \"Select a binding:\",\n"
"        items = items,\n"
"        submit = function (i)\n"
"            mp.command(items[i]:gsub(\"^.- \", \"\"))\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"local properties = {}\n"
"\n"
"local function add_property(property, value)\n"
"    value = value or mp.get_property_native(property)\n"
"\n"
"    if type(value) == \"table\" and next(value) then\n"
"        for key, val in pairs(value) do\n"
"            add_property(property .. \"/\" .. key, val)\n"
"        end\n"
"    else\n"
"        properties[#properties + 1] = property .. \": \" .. utils.to_string(value)\n"
"    end\n"
"end\n"
"\n"
"mp.add_key_binding(nil, \"show-properties\", function ()\n"
"    properties = {}\n"
"\n"
"    -- Don't log errors for renamed and removed properties.\n"
"    local msg_level_backup = mp.get_property(\"msg-level\")\n"
"    mp.set_property(\"msg-level\", msg_level_backup == \"\" and \"cplayer=no\"\n"
"                                 or msg_level_backup .. \",cplayer=no\")\n"
"\n"
"    for _, property in pairs(mp.get_property_native(\"property-list\")) do\n"
"        add_property(property)\n"
"    end\n"
"\n"
"    mp.set_property(\"msg-level\", msg_level_backup)\n"
"\n"
"    add_property(\"current-tracks/audio\")\n"
"    add_property(\"current-tracks/video\")\n"
"    add_property(\"current-tracks/sub\")\n"
"    add_property(\"current-tracks/sub2\")\n"
"\n"
"    table.sort(properties)\n"
"\n"
"    input.select({\n"
"        prompt = \"Inspect a property:\",\n"
"        items = properties,\n"
"        submit = function (i)\n"
"            if mp.get_property_native(\"vo-configured\") then\n"
"                mp.commandv(\"expand-properties\", \"show-text\",\n"
"                            (#properties[i] > 100 and\n"
"                             \"\044{osd-ass-cc/0}{\\\\fs9}\044{osd-ass-cc/1}\" or \"\") ..\n"
"                            \"\044>\" .. properties[i], 20000)\n"
"            else\n"
"                mp.msg.info(properties[i])\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
"\n"
"mp.add_key_binding(nil, \"menu\", function ()\n"
"    local sub_track_count = 0\n"
"    local audio_track_count = 0\n"
"    local video_track_count = 0\n"
"    local text_sub_selected = false\n"
"    local is_disc = mp.get_property(\"current-demuxer\") == \"disc\"\n"
"\n"
"    local image_sub_codecs = {[\"dvd_subtitle\"] = true, [\"hdmv_pgs_subtitle\"] = true}\n"
"\n"
"    for _, track in pairs(mp.get_property_native(\"track-list\")) do\n"
"        if track.type == \"sub\" then\n"
"            sub_track_count = sub_track_count + 1\n"
"\n"
"            if track[\"main-selection\"] == 0 and not image_sub_codecs[track.codec] then\n"
"                text_sub_selected = true\n"
"            end\n"
"        elseif track.type == \"audio\" then\n"
"            audio_track_count = audio_track_count + 1\n"
"        elseif track.type == \"video\" then\n"
"            video_track_count = video_track_count + 1\n"
"        end\n"
"    end\n"
"\n"
"    local menu = {\n"
"        {\"Subtitles\", \"script-binding select/select-sid\", sub_track_count > 0},\n"
"        {\"Secondary subtitles\", \"script-binding select/select-secondary-sid\", sub_track_count > 1},\n"
"        {\"Subtitle lines\", \"script-binding select/select-subtitle-line\", text_sub_selected},\n"
"        {\"Audio tracks\", \"script-binding select/select-aid\", audio_track_count > 1},\n"
"        {\"Video tracks\", \"script-binding select/select-vid\", video_track_count > 1},\n"
"        {\"Playlist\", \"script-binding select/select-playlist\",\n"
"         mp.get_property_native(\"playlist-count\") > 1},\n"
"        {\"Chapters\", \"script-binding select/select-chapter\", mp.get_property(\"chapter\")},\n"
"        {is_disc and \"Titles\" or \"Editions\", \"script-binding select/select-edition\",\n"
"         mp.get_property_native(\"edition-list/count\", 0) > 1},\n"
"        {\"Audio devices\", \"script-binding select/select-audio-device\", audio_track_count > 0},\n"
"        {\"Key bindings\", \"script-binding select/select-binding\", true},\n"
"        {\"History\", \"script-binding select/select-watch-history\", true},\n"
"        {\"Watch later\", \"script-binding select/select-watch-later\", true},\n"
"        {\"Stats for nerds\", \"script-binding stats/display-page-1-toggle\", true},\n"
"        {\"File info\", \"script-binding stats/display-page-5-toggle\", mp.get_property(\"filename\")},\n"
"        {\"Help\", \"script-binding stats/display-page-4-toggle\", true},\n"
"    }\n"
"\n"
"    local labels = {}\n"
"    local commands = {}\n"
"\n"
"    for _, entry in ipairs(menu) do\n"
"        if entry[3] then\n"
"            labels[#labels + 1] = entry[1]\n"
"            commands[#commands + 1] = entry[2]\n"
"        end\n"
"    end\n"
"\n"
"    input.select({\n"
"        prompt = \"\",\n"
"        items = labels,\n"
"        keep_open = true,\n"
"        submit = function (i)\n"
"            mp.command(commands[i])\n"
"\n"
"            if not commands[i]:find(\"^script%-binding select/\") then\n"
"                input.terminate()\n"
"            end\n"
"        end,\n"
"    })\n"
"end)\n"
